import dingosdk

from dingodb.sdk_client import SDKClient
from dingodb.common.document_rep import (
    DocumentType,
    DocumentColumn,
    DocumentSchema,
    AddResult,
    DocSearchResult,
    DocQueryResult,
    DocScanQueryResult,
    DocIndexMetricsResult,
    DocDeleteResult
)

from .sdk_document_param import (
    CreateIndexParam,
    DocumentDeleteParam,
    DocumentAddParam,
    DocumentSearchParam,
    DocumentQueryParam,
    DocumentGetBorderParam,
    DocumentScanQueryParam,
    DocumentCountParam
)

from .sdk_document_adapter import (
    document_doc_with_id_to_doc_with_id,
    document_add_result_to_add_result,
    document_search_result_to_search_result,
    document_scan_query_result_to_scan_query_result,
    document_get_index_metrics_to_get_index_metrics,
    document_delete_result_to_delete_result
)

from typing import List, Dict

scalar_type_to_document_type = {
    DocumentType.BOOL: dingosdk.Type.kBOOL,
    DocumentType.INT64: dingosdk.Type.kINT64,
    DocumentType.DOUBLE: dingosdk.Type.kDOUBLE,
    DocumentType.STRING: dingosdk.Type.kSTRING,
    DocumentType.BYTES: dingosdk.Type.kBYTES,
}

document_type_to_scalar_type = {
    dingosdk.Type.kBOOL: DocumentType.BOOL,
    dingosdk.Type.kINT64: DocumentType.INT64,
    dingosdk.Type.kDOUBLE: DocumentType.DOUBLE,
    dingosdk.Type.kSTRING: DocumentType.STRING,
    dingosdk.Type.kBYTES: DocumentType.BYTES,
}


class SDKDocumentClient:
    def __init__(self, client: SDKClient):
        """
        __init__ init Client

        Args:
            client: SDKClient
        """
        self.schema_id = 2
        self.client = client.dingosdk_client

        s, self.document_client = self.client.NewDocumentClient()
        if not s.ok():
            raise RuntimeError(f"dongo document client build fail: {s.ToString()}")

    def create_index(
            self, param: CreateIndexParam, schema: DocumentSchema = None
    ) -> bool:
        """
        create_index create index

        Args:
            param (CreateIndexParam): create index param
            schema (DocumentSchema): scalar data schema

        Returns:
            bool: create index result
        """
        s, creator = self.client.NewDocumentIndexCreator()
        assert s.ok(), f"dingo creator build fail: {s.ToString()}"
        creator.SetSchemaId(self.schema_id)
        creator.SetName(param.index_name)
        creator.SetReplicaNum(param.replicas)
        if param.operand is not None:
            if len(param.operand) != 0:
                creator.SetRangePartitions(param.operand)

        if param.json_params != "":
            creator.SetJsonParams(param.json_params)

        if param.start_id:
            creator.SetAutoIncrementStart(param.start_id)

        if schema is not None and len(schema.cols) != 0:
            sdk_scalar_schem = dingosdk.DocumentSchema()

            for col in schema.cols:
                sdk_col = dingosdk.DocumentColumn(
                    col.key, scalar_type_to_document_type[col.type]
                )

                sdk_scalar_schem.AddColumn(sdk_col)

            creator.SetSchema(sdk_scalar_schem)

        s, index_id = creator.Create()
        if s.ok():
            return True
        else:
            raise RuntimeError(
                f"create index {param.index_name} fail: {s.ToString()}, index_id: {index_id}"
            )

    def delete_index(self, index_name: str) -> bool:
        """
        delete_index del/drop index

        Args:
            index_name (str): the name of index

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """
        s = self.client.DropDocumentIndexByName(self.schema_id, index_name)
        if s.ok():
            return True
        else:
            raise RuntimeError(f"delete index {index_name} fail: {s.ToString()}")

    def get_schema(self, index_name: str) -> DocumentSchema:
        """
        get_schema get dschema

        Args:
            index_name (str): the name of index

        Raises:
            RuntimeError: return error

        Returns:
            DocumentSchema: dingodb.common.document_rep.DocumentSchema
        """
        status, out_doc_index = self.client.GetDocumentIndex(self.schema_id, index_name)
        if not status.ok():
            raise RuntimeError(
                f"get index {index_name} error: {status.ToString()}"
            )
        else:
            schema_dict = out_doc_index.GetSchema()

        document_schema = DocumentSchema()
        for key, value in schema_dict.items():
            document_column = DocumentColumn(key, document_type_to_scalar_type[value])
            document_schema.add_document_column(document_column)

        return document_schema


    def document_add(self, add_param: DocumentAddParam) -> AddResult:
        """
        document_add add document

        Args:
            add_param: DocumentAddParam

        Raises:
            RuntimeError: return error

        Returns:
            AddResult: dingodb.common.document_rep.AddResult
        """
        status, out_doc_index = self.client.GetDocumentIndex(self.schema_id, add_param.index_name)
        if not status.ok():
            raise RuntimeError(
                f"get index {add_param.index_name} error: {status.ToString()}"
            )
        else:
            schema_dict = out_doc_index.GetSchema()

        documents = []
        for i, v in enumerate(add_param.documents):
            id = 0
            if add_param.ids is not None:
                id = add_param.ids[i]

            tmp_document = dingosdk.Document()
            for key, value in add_param.documents[i].items():
                if key not in schema_dict:
                    raise KeyError(f"The key '{key}' is not in the schema")

                if schema_dict[key] == dingosdk.Type.kINT64:
                    tmp_document.AddField(key, dingosdk.DocValue.FromInt(value))
                elif schema_dict[key] == dingosdk.Type.kDOUBLE:
                    tmp_document.AddField(key, dingosdk.DocValue.FromDouble(value))
                elif schema_dict[key] == dingosdk.Type.kSTRING:
                    tmp_document.AddField(key, dingosdk.DocValue.FromString(value))
                elif schema_dict[key] == dingosdk.Type.kBYTES:
                    tmp_document.AddField(key, dingosdk.DocValue.FromBytes(value))
                elif schema_dict[key] == dingosdk.Type.kBOOL:
                    tmp_document.AddField(key, dingosdk.DocValue.FromBool(value))
                else:
                    raise RuntimeError(f"not support type: {schema_dict[key]}")

            document_with_id = dingosdk.DocWithId(id, tmp_document)
            documents.append(document_with_id)

        s, documents = self.document_client.AddByIndexName(
            self.schema_id, add_param.index_name, documents
        )

        if s.ok():
            return document_add_result_to_add_result(documents)

        else:
            raise RuntimeError(
                f"add document in {add_param.index_name} fail: {s.ToString()}"
            )

    def document_search(self, param: DocumentSearchParam) -> DocSearchResult:
        """
        document_search search document

        Args:
            param : DocumentSearchParam

        Raises:
            RuntimeError: return error

        Returns:
            DocSearchResult: dingodb.common.document_rep.DocSearchResult
        """

        document_param = dingosdk.DocSearchParam()
        document_param.query_string = param.query_string
        document_param.top_n = param.top_n

        document_param.use_id_filter = param.use_id_filter
        if param.use_id_filter:
            document_param.doc_ids = param.doc_ids

        document_param.with_scalar_data = param.with_scalar_data

        if param.column_names:
            document_param.column_names = param.column_names
        if param.selected_keys:
            document_param.selected_keys = param.selected_keys

        s, result = self.document_client.SearchByIndexName(
            self.schema_id, param.index_name, document_param
        )

        if s.ok():
            # print(result.ToString())
            return document_search_result_to_search_result(result)
        else:
            raise RuntimeError(f"search index:{param.index_name} fail: {s.ToString()}")

    def document_query(self, param: DocumentQueryParam) -> DocQueryResult:
        """
        document_query query document

        Args:
            param : DocumentQueryParam

        Raises:
            RuntimeError: return error

        Returns:
            DocQueryResult: dingodb.common.document_rep.DocQueryResult
        """
        document_param = dingosdk.DocQueryParam()
        document_param.doc_ids = param.doc_ids
        document_param.with_scalar_data = param.with_scalar_data
        if param.selected_keys:
            document_param.selected_keys = param.selected_keys

        s, result = self.document_client.BatchQueryByIndexName(
            self.schema_id, param.index_name, document_param
        )

        if s.ok():
            query_results = {
                doc.id: document_doc_with_id_to_doc_with_id(doc) for doc in result.docs
            }

            return_result = []
            for id in param.doc_ids:
                if id in query_results:
                    return_result.append(query_results[id])
                else:
                    return_result.append(None)

            return DocQueryResult(return_result)
        else:
            raise RuntimeError(f"query index:{param.index_name} fail: {s.ToString()}")

    def document_get_border(self, param: DocumentGetBorderParam) -> int:
        """
        document_get_border get_border document

        Args:
            param : DocumentGetBorderParam

        Raises:
            RuntimeError: return error

        Returns:
            List[dict]: get_border results
        """
        s, result = self.document_client.GetBorderByIndexName(
            self.schema_id, param.index_name, param.is_max
        )

        if s.ok():
            return result
        else:
            raise RuntimeError(f"get_border index:{param.index_name} fail: {s.ToString()}")

    def document_scan_query(self, param: DocumentScanQueryParam) -> DocScanQueryResult:
        """
        document_scan_query scan_query document

        Args:
            param : DocumentScanQueryParam

        Raises:
            RuntimeError: return error

        Returns:
            DocScanQueryResult: dingodb.common.document_rep.DocScanQueryResult
        """
        document_param = dingosdk.DocScanQueryParam()
        document_param.doc_id_start = param.doc_id_start
        document_param.doc_id_end = param.doc_id_end
        document_param.is_reverse = param.is_reverse
        document_param.max_scan_count = param.max_scan_count

        document_param.with_scalar_data = param.with_scalar_data
        if param.selected_keys:
            document_param.selected_keys = param.selected_keys

        s, result = self.document_client.ScanQueryByIndexName(
            self.schema_id, param.index_name, document_param
        )

        if s.ok():
            # print(result.ToString())
            return document_scan_query_result_to_scan_query_result(result)
        else:
            raise RuntimeError(f"scan_query index:{param.index_name} fail: {s.ToString()}")

    def document_index_metrics(self, index_name: str) -> DocIndexMetricsResult:
        """
        document_index_metrics index_metrics in index

        Args:
            index_name (str): the name of in index

        Raises:
            RuntimeError: return error

        Returns:
            DocIndexMetricsResult: dingodb.common.document_rep.DocIndexMetricsResult
        """
        s, result = self.document_client.GetIndexMetricsByIndexName(
            self.schema_id, index_name
        )
        if s.ok():
            # print(result.ToString())
            return document_get_index_metrics_to_get_index_metrics(result)
        else:
            raise RuntimeError(f"get index {index_name} metrics fail: {s.ToString()}")

    def document_count(self, param: DocumentCountParam) -> int:
        """
        document_count count document

        Args:
            param : DocumentCountParam

        Raises:
            RuntimeError: return error

        Returns:
            int: count int
        """
        s, result = self.document_client.CountByIndexName(
            self.schema_id, param.index_name, param.doc_id_start, param.doc_id_end
        )

        if s.ok():
            return result
        else:
            raise RuntimeError(f"count index:{param.index_name} fail: {s.ToString()}")

    def document_delete(self, param: DocumentDeleteParam) -> List[DocDeleteResult]:
        """
        document_delete delete document with ids

        Args:
            params: DocumentDeleteParam

        Raises:
            RuntimeError: return error

        Returns:
            List[DocDeleteResult] : dingodb.common.document_rep.DocDeleteResult
        """
        s, result = self.document_client.DeleteByIndexName(
            self.schema_id, param.index_name, param.ids
        )

        if s.ok():
            delete_result = {
                doc.doc_id: document_delete_result_to_delete_result(doc) for doc in result
            }

            return_result = {}
            for id in param.ids:
                if id in delete_result:
                    return_result[id] = delete_result[id]
                else:
                    return_result[id] = None

            return [delete_result for delete_result in return_result.values()]
        else:
            raise RuntimeError(
                f"document delete form index:{param.index_name} fail: {s.ToString()}"
            )

    def document_get_auto_increment_memory(self, index_name: str) -> int:
        """
        document_get_auto_increment_id get_auto_increment_id

        Args:
           index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            auto_increment_id
        """
        s, result = self.document_client.GetAutoIncrementIdByIndexName(
            self.schema_id, index_name
        )
        if s.ok():
            return result
        else:
            raise RuntimeError(
                f"document count memory form index:{index_name} fail: {s.ToString()}"
            )

    def document_update_auto_increment_id(self, index_name: str, start_id: int):
        """
        document_update_auto_increment_id update AutoIncrementId

        Args:
           index_name: str

        Raises:
            RuntimeError: return error

        Returns:

        """
        s = self.document_client.UpdateAutoIncrementIdByIndexName(
            self.schema_id, index_name, start_id
        )
        if not s.ok():
            raise RuntimeError(
                f"document count memory form index:{index_name} fail: {s.ToString()}"
            )



import proxy_common_pb2 as _proxy_common_pb2
import proxy_error_pb2 as _proxy_error_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class VectorAddRequest(_message.Message):
    __slots__ = ["schema_name", "index_name", "vectors", "replace_deleted", "is_update"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    VECTORS_FIELD_NUMBER: _ClassVar[int]
    REPLACE_DELETED_FIELD_NUMBER: _ClassVar[int]
    IS_UPDATE_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    vectors: _containers.RepeatedCompositeFieldContainer[_proxy_common_pb2.VectorWithId]
    replace_deleted: bool
    is_update: bool
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ..., vectors: _Optional[_Iterable[_Union[_proxy_common_pb2.VectorWithId, _Mapping]]] = ..., replace_deleted: bool = ..., is_update: bool = ...) -> None: ...

class VectorAddResponse(_message.Message):
    __slots__ = ["error", "vectors"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    VECTORS_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    vectors: _containers.RepeatedCompositeFieldContainer[_proxy_common_pb2.VectorWithId]
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., vectors: _Optional[_Iterable[_Union[_proxy_common_pb2.VectorWithId, _Mapping]]] = ...) -> None: ...

class VectorGetRequest(_message.Message):
    __slots__ = ["schema_name", "index_name", "vector_ids", "without_vector_data", "without_scalar_data", "selected_keys", "without_table_data"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    VECTOR_IDS_FIELD_NUMBER: _ClassVar[int]
    WITHOUT_VECTOR_DATA_FIELD_NUMBER: _ClassVar[int]
    WITHOUT_SCALAR_DATA_FIELD_NUMBER: _ClassVar[int]
    SELECTED_KEYS_FIELD_NUMBER: _ClassVar[int]
    WITHOUT_TABLE_DATA_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    vector_ids: _containers.RepeatedScalarFieldContainer[int]
    without_vector_data: bool
    without_scalar_data: bool
    selected_keys: _containers.RepeatedScalarFieldContainer[str]
    without_table_data: bool
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ..., vector_ids: _Optional[_Iterable[int]] = ..., without_vector_data: bool = ..., without_scalar_data: bool = ..., selected_keys: _Optional[_Iterable[str]] = ..., without_table_data: bool = ...) -> None: ...

class VectorGetResponse(_message.Message):
    __slots__ = ["error", "vectors"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    VECTORS_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    vectors: _containers.RepeatedCompositeFieldContainer[_proxy_common_pb2.VectorWithId]
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., vectors: _Optional[_Iterable[_Union[_proxy_common_pb2.VectorWithId, _Mapping]]] = ...) -> None: ...

class VectorSearchRequest(_message.Message):
    __slots__ = ["schema_name", "index_name", "vectors", "parameter"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    VECTORS_FIELD_NUMBER: _ClassVar[int]
    PARAMETER_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    vectors: _containers.RepeatedCompositeFieldContainer[_proxy_common_pb2.VectorWithId]
    parameter: _proxy_common_pb2.VectorSearchParameter
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ..., vectors: _Optional[_Iterable[_Union[_proxy_common_pb2.VectorWithId, _Mapping]]] = ..., parameter: _Optional[_Union[_proxy_common_pb2.VectorSearchParameter, _Mapping]] = ...) -> None: ...

class VectorWithDistanceResult(_message.Message):
    __slots__ = ["vector_with_distances"]
    VECTOR_WITH_DISTANCES_FIELD_NUMBER: _ClassVar[int]
    vector_with_distances: _containers.RepeatedCompositeFieldContainer[_proxy_common_pb2.VectorWithDistance]
    def __init__(self, vector_with_distances: _Optional[_Iterable[_Union[_proxy_common_pb2.VectorWithDistance, _Mapping]]] = ...) -> None: ...

class VectorSearchResponse(_message.Message):
    __slots__ = ["error", "batch_results"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    BATCH_RESULTS_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    batch_results: _containers.RepeatedCompositeFieldContainer[VectorWithDistanceResult]
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., batch_results: _Optional[_Iterable[_Union[VectorWithDistanceResult, _Mapping]]] = ...) -> None: ...

class VectorDeleteRequest(_message.Message):
    __slots__ = ["schema_name", "index_name", "ids"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    IDS_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    ids: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ..., ids: _Optional[_Iterable[int]] = ...) -> None: ...

class VectorDeleteResponse(_message.Message):
    __slots__ = ["error", "key_states"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    KEY_STATES_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    key_states: _containers.RepeatedScalarFieldContainer[bool]
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., key_states: _Optional[_Iterable[bool]] = ...) -> None: ...

class VectorGetBorderIdRequest(_message.Message):
    __slots__ = ["schema_name", "index_name", "get_min"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    GET_MIN_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    get_min: bool
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ..., get_min: bool = ...) -> None: ...

class VectorGetBorderIdResponse(_message.Message):
    __slots__ = ["error", "id"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    ID_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    id: int
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., id: _Optional[int] = ...) -> None: ...

class VectorScanQueryRequest(_message.Message):
    __slots__ = ["schema_name", "index_name", "vector_id_start", "is_reverse_scan", "max_scan_count", "vector_id_end", "without_vector_data", "without_scalar_data", "selected_keys", "without_table_data", "use_scalar_filter", "scalar_for_filter"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    VECTOR_ID_START_FIELD_NUMBER: _ClassVar[int]
    IS_REVERSE_SCAN_FIELD_NUMBER: _ClassVar[int]
    MAX_SCAN_COUNT_FIELD_NUMBER: _ClassVar[int]
    VECTOR_ID_END_FIELD_NUMBER: _ClassVar[int]
    WITHOUT_VECTOR_DATA_FIELD_NUMBER: _ClassVar[int]
    WITHOUT_SCALAR_DATA_FIELD_NUMBER: _ClassVar[int]
    SELECTED_KEYS_FIELD_NUMBER: _ClassVar[int]
    WITHOUT_TABLE_DATA_FIELD_NUMBER: _ClassVar[int]
    USE_SCALAR_FILTER_FIELD_NUMBER: _ClassVar[int]
    SCALAR_FOR_FILTER_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    vector_id_start: int
    is_reverse_scan: bool
    max_scan_count: int
    vector_id_end: int
    without_vector_data: bool
    without_scalar_data: bool
    selected_keys: _containers.RepeatedScalarFieldContainer[str]
    without_table_data: bool
    use_scalar_filter: bool
    scalar_for_filter: _proxy_common_pb2.VectorScalarData
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ..., vector_id_start: _Optional[int] = ..., is_reverse_scan: bool = ..., max_scan_count: _Optional[int] = ..., vector_id_end: _Optional[int] = ..., without_vector_data: bool = ..., without_scalar_data: bool = ..., selected_keys: _Optional[_Iterable[str]] = ..., without_table_data: bool = ..., use_scalar_filter: bool = ..., scalar_for_filter: _Optional[_Union[_proxy_common_pb2.VectorScalarData, _Mapping]] = ...) -> None: ...

class VectorScanQueryResponse(_message.Message):
    __slots__ = ["error", "vectors"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    VECTORS_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    vectors: _containers.RepeatedCompositeFieldContainer[_proxy_common_pb2.VectorWithId]
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., vectors: _Optional[_Iterable[_Union[_proxy_common_pb2.VectorWithId, _Mapping]]] = ...) -> None: ...

class VectorGetRegionMetricsRequest(_message.Message):
    __slots__ = ["schema_name", "index_name"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ...) -> None: ...

class VectorGetRegionMetricsResponse(_message.Message):
    __slots__ = ["error", "metrics"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    METRICS_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    metrics: _proxy_common_pb2.VectorIndexMetrics
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., metrics: _Optional[_Union[_proxy_common_pb2.VectorIndexMetrics, _Mapping]] = ...) -> None: ...

class VectorCountRequest(_message.Message):
    __slots__ = ["schema_name", "index_name"]
    SCHEMA_NAME_FIELD_NUMBER: _ClassVar[int]
    INDEX_NAME_FIELD_NUMBER: _ClassVar[int]
    schema_name: str
    index_name: str
    def __init__(self, schema_name: _Optional[str] = ..., index_name: _Optional[str] = ...) -> None: ...

class VectorCountResponse(_message.Message):
    __slots__ = ["error", "count"]
    ERROR_FIELD_NUMBER: _ClassVar[int]
    COUNT_FIELD_NUMBER: _ClassVar[int]
    error: _proxy_error_pb2.Error
    count: int
    def __init__(self, error: _Optional[_Union[_proxy_error_pb2.Error, _Mapping]] = ..., count: _Optional[int] = ...) -> None: ...



from pydantic import BaseModel, validator
from typing import List, Dict, Union


class CreateIndexParam(BaseModel):
    index_name: str
    replicas: int = 3
    operand: List[int] = None
    start_id: int = 0
    json_params: str = ""

    @validator("replicas", always=True)
    def check_replicas(cls, value):
        if value < 0:
            raise ValueError(f"{value} must >= 0")
        if value == 0:
            value = 3
        return value

    @validator("start_id", always=True)
    def check_start_id(cls, value):
        if value < 0:
            raise ValueError(f"{value} must >= 0")
        return value

    @validator("json_params", always=True)
    def check_json_params(cls, value):
        return value


class DocumentDeleteParam(BaseModel):
    index_name: str
    ids: List[int]

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        assert len(id_list) > 0, f"ids list length must > 0, but get {id_list}"
        return id_list


class DocumentAddParam(BaseModel):
    index_name: str
    documents: List
    ids: List[int] = None

    @validator("documents", always=True)
    def check_documents(cls, value):
        return value

    @validator("ids", pre=True, always=True)
    def check_ids(cls, value, values):
        if value is not None:
            assert (
                    len(values.get("documents")) == len(value)
            ), f"length documents:{len(values.get('documents'))} ids:{len(value)} is not equal"
            for id in value:
                if id <= 0:
                    raise ValueError("id must > 0")
        return value


class DocumentSearchParam(BaseModel):
    index_name: str
    query_string: str
    top_n: int = 0
    use_id_filter: bool = False
    doc_ids: List[int] = None
    with_scalar_data: bool = False
    column_names: List[str] = None
    selected_keys: List[str] = None

    @validator("top_n", pre=True, always=True)
    def check_top_k(cls, value, field):
        if not value > 0:
            raise ValueError(f"{field.name} must > 0")
        return value

    @validator("doc_ids", always=True)
    def check_doc_ids(cls, value, values):
        if value is None or value == []:
            value = []
            return value
        else:
            values["use_id_filter"] = True
            return value

    @validator("column_names", always=True)
    def check_column_names(cls, value):
        if value is None:
            value = []
        return value

    @validator("selected_keys", always=True)
    def check_selected_keys(cls, value):
        if value is None:
            value = []
        return value


class DocumentQueryParam(BaseModel):
    index_name: str
    doc_ids: List[int]
    with_scalar_data: bool = False
    selected_keys: List[str] = None
    
    @validator("doc_ids", always=True)
    def check_doc_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        assert len(id_list) > 0, f"doc_ids list length must > 0, but get {id_list}"
        return id_list

    @validator("selected_keys", always=True)
    def check_selected_keys(cls, value):
        if value is None:
            value = []
        return value


class DocumentGetBorderParam(BaseModel):
    index_name: str
    is_max: bool = True


class DocumentScanQueryParam(BaseModel):
    index_name: str
    doc_id_start: int
    doc_id_end: int = 0
    is_reverse: bool = False
    max_scan_count: int = 0
    with_scalar_data: bool = True
    selected_keys: List[str] = None

    @validator("is_reverse", always=True)
    def check_is_reverse(cls, value, values):
        if value:
            if values.get("doc_id_start") <= values.get("doc_id_end"):
                raise ValueError("doc_id_end must be less than doc_id_start in reverse scan")
        elif not value:
            if values.get("doc_id_start") >= values.get("doc_id_end") and values.get("doc_id_end") != 0:
                raise ValueError("doc_id_end must be greater than doc_id_start in forward scan")
        return value

    @validator("selected_keys", always=True)
    def check_selected_keys(cls, value):
        if value is None:
            value = []
        return value


class DocumentCountParam(BaseModel):
    index_name: str
    doc_id_start: int
    doc_id_end: int

    @validator("doc_id_end", always=True)
    def check_doc_id_end(cls, value, values):
        if value <= values.get("doc_id_start"):
            raise ValueError(f"doc_id_end must > doc_id_start")
        return value


from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class ValueType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    FLOAT: _ClassVar[ValueType]
    UINT8: _ClassVar[ValueType]

class ScalarIndexType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    SCALAR_INDEX_TYPE_NONE: _ClassVar[ScalarIndexType]
    SCALAR_INDEX_TYPE_LSM: _ClassVar[ScalarIndexType]
    SCALAR_INDEX_TYPE_BTREE: _ClassVar[ScalarIndexType]

class IndexType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    INDEX_TYPE_NONE: _ClassVar[IndexType]
    INDEX_TYPE_VECTOR: _ClassVar[IndexType]
    INDEX_TYPE_SCALAR: _ClassVar[IndexType]

class VectorIndexType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    VECTOR_INDEX_TYPE_NONE: _ClassVar[VectorIndexType]
    VECTOR_INDEX_TYPE_FLAT: _ClassVar[VectorIndexType]
    VECTOR_INDEX_TYPE_IVF_FLAT: _ClassVar[VectorIndexType]
    VECTOR_INDEX_TYPE_IVF_PQ: _ClassVar[VectorIndexType]
    VECTOR_INDEX_TYPE_HNSW: _ClassVar[VectorIndexType]
    VECTOR_INDEX_TYPE_DISKANN: _ClassVar[VectorIndexType]
    VECTOR_INDEX_TYPE_BRUTEFORCE: _ClassVar[VectorIndexType]

class MetricType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    METRIC_TYPE_NONE: _ClassVar[MetricType]
    METRIC_TYPE_L2: _ClassVar[MetricType]
    METRIC_TYPE_INNER_PRODUCT: _ClassVar[MetricType]
    METRIC_TYPE_COSINE: _ClassVar[MetricType]

class VectorFilter(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    SCALAR_FILTER: _ClassVar[VectorFilter]
    TABLE_FILTER: _ClassVar[VectorFilter]
    VECTOR_ID_FILTER: _ClassVar[VectorFilter]

class VectorFilterType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    QUERY_POST: _ClassVar[VectorFilterType]
    QUERY_PRE: _ClassVar[VectorFilterType]

class ScalarFieldType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    NONE: _ClassVar[ScalarFieldType]
    BOOL: _ClassVar[ScalarFieldType]
    INT8: _ClassVar[ScalarFieldType]
    INT16: _ClassVar[ScalarFieldType]
    INT32: _ClassVar[ScalarFieldType]
    INT64: _ClassVar[ScalarFieldType]
    FLOAT32: _ClassVar[ScalarFieldType]
    DOUBLE: _ClassVar[ScalarFieldType]
    STRING: _ClassVar[ScalarFieldType]
    BYTES: _ClassVar[ScalarFieldType]
FLOAT: ValueType
UINT8: ValueType
SCALAR_INDEX_TYPE_NONE: ScalarIndexType
SCALAR_INDEX_TYPE_LSM: ScalarIndexType
SCALAR_INDEX_TYPE_BTREE: ScalarIndexType
INDEX_TYPE_NONE: IndexType
INDEX_TYPE_VECTOR: IndexType
INDEX_TYPE_SCALAR: IndexType
VECTOR_INDEX_TYPE_NONE: VectorIndexType
VECTOR_INDEX_TYPE_FLAT: VectorIndexType
VECTOR_INDEX_TYPE_IVF_FLAT: VectorIndexType
VECTOR_INDEX_TYPE_IVF_PQ: VectorIndexType
VECTOR_INDEX_TYPE_HNSW: VectorIndexType
VECTOR_INDEX_TYPE_DISKANN: VectorIndexType
VECTOR_INDEX_TYPE_BRUTEFORCE: VectorIndexType
METRIC_TYPE_NONE: MetricType
METRIC_TYPE_L2: MetricType
METRIC_TYPE_INNER_PRODUCT: MetricType
METRIC_TYPE_COSINE: MetricType
SCALAR_FILTER: VectorFilter
TABLE_FILTER: VectorFilter
VECTOR_ID_FILTER: VectorFilter
QUERY_POST: VectorFilterType
QUERY_PRE: VectorFilterType
NONE: ScalarFieldType
BOOL: ScalarFieldType
INT8: ScalarFieldType
INT16: ScalarFieldType
INT32: ScalarFieldType
INT64: ScalarFieldType
FLOAT32: ScalarFieldType
DOUBLE: ScalarFieldType
STRING: ScalarFieldType
BYTES: ScalarFieldType

class ColumnDefinition(_message.Message):
    __slots__ = ["name", "sql_type", "element_type", "precision", "scale", "nullable", "indexOfKey", "has_default_val", "default_val"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    SQL_TYPE_FIELD_NUMBER: _ClassVar[int]
    ELEMENT_TYPE_FIELD_NUMBER: _ClassVar[int]
    PRECISION_FIELD_NUMBER: _ClassVar[int]
    SCALE_FIELD_NUMBER: _ClassVar[int]
    NULLABLE_FIELD_NUMBER: _ClassVar[int]
    INDEXOFKEY_FIELD_NUMBER: _ClassVar[int]
    HAS_DEFAULT_VAL_FIELD_NUMBER: _ClassVar[int]
    DEFAULT_VAL_FIELD_NUMBER: _ClassVar[int]
    name: str
    sql_type: str
    element_type: str
    precision: int
    scale: int
    nullable: bool
    indexOfKey: int
    has_default_val: bool
    default_val: str
    def __init__(self, name: _Optional[str] = ..., sql_type: _Optional[str] = ..., element_type: _Optional[str] = ..., precision: _Optional[int] = ..., scale: _Optional[int] = ..., nullable: bool = ..., indexOfKey: _Optional[int] = ..., has_default_val: bool = ..., default_val: _Optional[str] = ...) -> None: ...

class Vector(_message.Message):
    __slots__ = ["dimension", "value_type", "float_values", "binary_values"]
    DIMENSION_FIELD_NUMBER: _ClassVar[int]
    VALUE_TYPE_FIELD_NUMBER: _ClassVar[int]
    FLOAT_VALUES_FIELD_NUMBER: _ClassVar[int]
    BINARY_VALUES_FIELD_NUMBER: _ClassVar[int]
    dimension: int
    value_type: ValueType
    float_values: _containers.RepeatedScalarFieldContainer[float]
    binary_values: _containers.RepeatedScalarFieldContainer[bytes]
    def __init__(self, dimension: _Optional[int] = ..., value_type: _Optional[_Union[ValueType, str]] = ..., float_values: _Optional[_Iterable[float]] = ..., binary_values: _Optional[_Iterable[bytes]] = ...) -> None: ...

class VectorScalarData(_message.Message):
    __slots__ = ["scalar_data"]
    class ScalarDataEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: ScalarValue
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[ScalarValue, _Mapping]] = ...) -> None: ...
    SCALAR_DATA_FIELD_NUMBER: _ClassVar[int]
    scalar_data: _containers.MessageMap[str, ScalarValue]
    def __init__(self, scalar_data: _Optional[_Mapping[str, ScalarValue]] = ...) -> None: ...

class VectorTableData(_message.Message):
    __slots__ = ["table_key", "table_value"]
    TABLE_KEY_FIELD_NUMBER: _ClassVar[int]
    TABLE_VALUE_FIELD_NUMBER: _ClassVar[int]
    table_key: bytes
    table_value: bytes
    def __init__(self, table_key: _Optional[bytes] = ..., table_value: _Optional[bytes] = ...) -> None: ...

class VectorWithId(_message.Message):
    __slots__ = ["id", "vector", "scalar_data", "table_data"]
    class ScalarDataEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: ScalarValue
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[ScalarValue, _Mapping]] = ...) -> None: ...
    ID_FIELD_NUMBER: _ClassVar[int]
    VECTOR_FIELD_NUMBER: _ClassVar[int]
    SCALAR_DATA_FIELD_NUMBER: _ClassVar[int]
    TABLE_DATA_FIELD_NUMBER: _ClassVar[int]
    id: int
    vector: Vector
    scalar_data: _containers.MessageMap[str, ScalarValue]
    table_data: VectorTableData
    def __init__(self, id: _Optional[int] = ..., vector: _Optional[_Union[Vector, _Mapping]] = ..., scalar_data: _Optional[_Mapping[str, ScalarValue]] = ..., table_data: _Optional[_Union[VectorTableData, _Mapping]] = ...) -> None: ...

class VectorWithDistance(_message.Message):
    __slots__ = ["id", "vector", "scalar_data", "distance", "metric_type"]
    class ScalarDataEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: ScalarValue
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[ScalarValue, _Mapping]] = ...) -> None: ...
    ID_FIELD_NUMBER: _ClassVar[int]
    VECTOR_FIELD_NUMBER: _ClassVar[int]
    SCALAR_DATA_FIELD_NUMBER: _ClassVar[int]
    DISTANCE_FIELD_NUMBER: _ClassVar[int]
    METRIC_TYPE_FIELD_NUMBER: _ClassVar[int]
    id: int
    vector: Vector
    scalar_data: _containers.MessageMap[str, ScalarValue]
    distance: float
    metric_type: MetricType
    def __init__(self, id: _Optional[int] = ..., vector: _Optional[_Union[Vector, _Mapping]] = ..., scalar_data: _Optional[_Mapping[str, ScalarValue]] = ..., distance: _Optional[float] = ..., metric_type: _Optional[_Union[MetricType, str]] = ...) -> None: ...

class IndexParameter(_message.Message):
    __slots__ = ["index_type", "vector_index_parameter", "scalar_index_parameter"]
    INDEX_TYPE_FIELD_NUMBER: _ClassVar[int]
    VECTOR_INDEX_PARAMETER_FIELD_NUMBER: _ClassVar[int]
    SCALAR_INDEX_PARAMETER_FIELD_NUMBER: _ClassVar[int]
    index_type: IndexType
    vector_index_parameter: VectorIndexParameter
    scalar_index_parameter: ScalarIndexParameter
    def __init__(self, index_type: _Optional[_Union[IndexType, str]] = ..., vector_index_parameter: _Optional[_Union[VectorIndexParameter, _Mapping]] = ..., scalar_index_parameter: _Optional[_Union[ScalarIndexParameter, _Mapping]] = ...) -> None: ...

class CreateFlatParam(_message.Message):
    __slots__ = ["dimension", "metric_type"]
    DIMENSION_FIELD_NUMBER: _ClassVar[int]
    METRIC_TYPE_FIELD_NUMBER: _ClassVar[int]
    dimension: int
    metric_type: MetricType
    def __init__(self, dimension: _Optional[int] = ..., metric_type: _Optional[_Union[MetricType, str]] = ...) -> None: ...

class CreateIvfFlatParam(_message.Message):
    __slots__ = ["dimension", "metric_type", "ncentroids"]
    DIMENSION_FIELD_NUMBER: _ClassVar[int]
    METRIC_TYPE_FIELD_NUMBER: _ClassVar[int]
    NCENTROIDS_FIELD_NUMBER: _ClassVar[int]
    dimension: int
    metric_type: MetricType
    ncentroids: int
    def __init__(self, dimension: _Optional[int] = ..., metric_type: _Optional[_Union[MetricType, str]] = ..., ncentroids: _Optional[int] = ...) -> None: ...

class CreateIvfPqParam(_message.Message):
    __slots__ = ["dimension", "metric_type", "ncentroids", "nsubvector", "bucket_init_size", "bucket_max_size", "nbits_per_idx"]
    DIMENSION_FIELD_NUMBER: _ClassVar[int]
    METRIC_TYPE_FIELD_NUMBER: _ClassVar[int]
    NCENTROIDS_FIELD_NUMBER: _ClassVar[int]
    NSUBVECTOR_FIELD_NUMBER: _ClassVar[int]
    BUCKET_INIT_SIZE_FIELD_NUMBER: _ClassVar[int]
    BUCKET_MAX_SIZE_FIELD_NUMBER: _ClassVar[int]
    NBITS_PER_IDX_FIELD_NUMBER: _ClassVar[int]
    dimension: int
    metric_type: MetricType
    ncentroids: int
    nsubvector: int
    bucket_init_size: int
    bucket_max_size: int
    nbits_per_idx: int
    def __init__(self, dimension: _Optional[int] = ..., metric_type: _Optional[_Union[MetricType, str]] = ..., ncentroids: _Optional[int] = ..., nsubvector: _Optional[int] = ..., bucket_init_size: _Optional[int] = ..., bucket_max_size: _Optional[int] = ..., nbits_per_idx: _Optional[int] = ...) -> None: ...

class CreateHnswParam(_message.Message):
    __slots__ = ["dimension", "metric_type", "efConstruction", "max_elements", "nlinks"]
    DIMENSION_FIELD_NUMBER: _ClassVar[int]
    METRIC_TYPE_FIELD_NUMBER: _ClassVar[int]
    EFCONSTRUCTION_FIELD_NUMBER: _ClassVar[int]
    MAX_ELEMENTS_FIELD_NUMBER: _ClassVar[int]
    NLINKS_FIELD_NUMBER: _ClassVar[int]
    dimension: int
    metric_type: MetricType
    efConstruction: int
    max_elements: int
    nlinks: int
    def __init__(self, dimension: _Optional[int] = ..., metric_type: _Optional[_Union[MetricType, str]] = ..., efConstruction: _Optional[int] = ..., max_elements: _Optional[int] = ..., nlinks: _Optional[int] = ...) -> None: ...

class CreateDiskAnnParam(_message.Message):
    __slots__ = ["dimension", "metric_type", "num_trees", "num_neighbors", "num_threads"]
    DIMENSION_FIELD_NUMBER: _ClassVar[int]
    METRIC_TYPE_FIELD_NUMBER: _ClassVar[int]
    NUM_TREES_FIELD_NUMBER: _ClassVar[int]
    NUM_NEIGHBORS_FIELD_NUMBER: _ClassVar[int]
    NUM_THREADS_FIELD_NUMBER: _ClassVar[int]
    dimension: int
    metric_type: MetricType
    num_trees: int
    num_neighbors: int
    num_threads: int
    def __init__(self, dimension: _Optional[int] = ..., metric_type: _Optional[_Union[MetricType, str]] = ..., num_trees: _Optional[int] = ..., num_neighbors: _Optional[int] = ..., num_threads: _Optional[int] = ...) -> None: ...

class CreateBruteForceParam(_message.Message):
    __slots__ = ["dimension", "metric_type"]
    DIMENSION_FIELD_NUMBER: _ClassVar[int]
    METRIC_TYPE_FIELD_NUMBER: _ClassVar[int]
    dimension: int
    metric_type: MetricType
    def __init__(self, dimension: _Optional[int] = ..., metric_type: _Optional[_Union[MetricType, str]] = ...) -> None: ...

class VectorIndexParameter(_message.Message):
    __slots__ = ["vector_index_type", "flat_parameter", "ivf_flat_parameter", "ivf_pq_parameter", "hnsw_parameter", "diskann_parameter", "bruteforce_parameter"]
    VECTOR_INDEX_TYPE_FIELD_NUMBER: _ClassVar[int]
    FLAT_PARAMETER_FIELD_NUMBER: _ClassVar[int]
    IVF_FLAT_PARAMETER_FIELD_NUMBER: _ClassVar[int]
    IVF_PQ_PARAMETER_FIELD_NUMBER: _ClassVar[int]
    HNSW_PARAMETER_FIELD_NUMBER: _ClassVar[int]
    DISKANN_PARAMETER_FIELD_NUMBER: _ClassVar[int]
    BRUTEFORCE_PARAMETER_FIELD_NUMBER: _ClassVar[int]
    vector_index_type: VectorIndexType
    flat_parameter: CreateFlatParam
    ivf_flat_parameter: CreateIvfFlatParam
    ivf_pq_parameter: CreateIvfPqParam
    hnsw_parameter: CreateHnswParam
    diskann_parameter: CreateDiskAnnParam
    bruteforce_parameter: CreateBruteForceParam
    def __init__(self, vector_index_type: _Optional[_Union[VectorIndexType, str]] = ..., flat_parameter: _Optional[_Union[CreateFlatParam, _Mapping]] = ..., ivf_flat_parameter: _Optional[_Union[CreateIvfFlatParam, _Mapping]] = ..., ivf_pq_parameter: _Optional[_Union[CreateIvfPqParam, _Mapping]] = ..., hnsw_parameter: _Optional[_Union[CreateHnswParam, _Mapping]] = ..., diskann_parameter: _Optional[_Union[CreateDiskAnnParam, _Mapping]] = ..., bruteforce_parameter: _Optional[_Union[CreateBruteForceParam, _Mapping]] = ...) -> None: ...

class VectorSchema(_message.Message):
    __slots__ = ["type", "is_key", "is_nullable", "index"]
    class Type(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = []
        BOOL: _ClassVar[VectorSchema.Type]
        INTEGER: _ClassVar[VectorSchema.Type]
        FLOAT: _ClassVar[VectorSchema.Type]
        LONG: _ClassVar[VectorSchema.Type]
        DOUBLE: _ClassVar[VectorSchema.Type]
        STRING: _ClassVar[VectorSchema.Type]
    BOOL: VectorSchema.Type
    INTEGER: VectorSchema.Type
    FLOAT: VectorSchema.Type
    LONG: VectorSchema.Type
    DOUBLE: VectorSchema.Type
    STRING: VectorSchema.Type
    TYPE_FIELD_NUMBER: _ClassVar[int]
    IS_KEY_FIELD_NUMBER: _ClassVar[int]
    IS_NULLABLE_FIELD_NUMBER: _ClassVar[int]
    INDEX_FIELD_NUMBER: _ClassVar[int]
    type: VectorSchema.Type
    is_key: bool
    is_nullable: bool
    index: int
    def __init__(self, type: _Optional[_Union[VectorSchema.Type, str]] = ..., is_key: bool = ..., is_nullable: bool = ..., index: _Optional[int] = ...) -> None: ...

class ScalarIndexParameter(_message.Message):
    __slots__ = ["scalar_index_type"]
    SCALAR_INDEX_TYPE_FIELD_NUMBER: _ClassVar[int]
    scalar_index_type: ScalarIndexType
    def __init__(self, scalar_index_type: _Optional[_Union[ScalarIndexType, str]] = ...) -> None: ...

class SearchFlatParam(_message.Message):
    __slots__ = ["parallel_on_queries"]
    PARALLEL_ON_QUERIES_FIELD_NUMBER: _ClassVar[int]
    parallel_on_queries: int
    def __init__(self, parallel_on_queries: _Optional[int] = ...) -> None: ...

class SearchIvfFlatParam(_message.Message):
    __slots__ = ["nprobe", "parallel_on_queries"]
    NPROBE_FIELD_NUMBER: _ClassVar[int]
    PARALLEL_ON_QUERIES_FIELD_NUMBER: _ClassVar[int]
    nprobe: int
    parallel_on_queries: int
    def __init__(self, nprobe: _Optional[int] = ..., parallel_on_queries: _Optional[int] = ...) -> None: ...

class SearchIvfPqParam(_message.Message):
    __slots__ = ["nprobe", "parallel_on_queries", "recall_num"]
    NPROBE_FIELD_NUMBER: _ClassVar[int]
    PARALLEL_ON_QUERIES_FIELD_NUMBER: _ClassVar[int]
    RECALL_NUM_FIELD_NUMBER: _ClassVar[int]
    nprobe: int
    parallel_on_queries: int
    recall_num: int
    def __init__(self, nprobe: _Optional[int] = ..., parallel_on_queries: _Optional[int] = ..., recall_num: _Optional[int] = ...) -> None: ...

class SearchHNSWParam(_message.Message):
    __slots__ = ["efSearch"]
    EFSEARCH_FIELD_NUMBER: _ClassVar[int]
    efSearch: int
    def __init__(self, efSearch: _Optional[int] = ...) -> None: ...

class SearchDiskAnnParam(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class VectorSearchParameter(_message.Message):
    __slots__ = ["top_n", "without_vector_data", "without_scalar_data", "selected_keys", "without_table_data", "flat", "ivf_flat", "ivf_pq", "hnsw", "diskann", "use_scalar_filter", "vector_filter", "vector_filter_type", "vector_coprocessor", "vector_ids", "use_brute_force", "langchain_expr"]
    TOP_N_FIELD_NUMBER: _ClassVar[int]
    WITHOUT_VECTOR_DATA_FIELD_NUMBER: _ClassVar[int]
    WITHOUT_SCALAR_DATA_FIELD_NUMBER: _ClassVar[int]
    SELECTED_KEYS_FIELD_NUMBER: _ClassVar[int]
    WITHOUT_TABLE_DATA_FIELD_NUMBER: _ClassVar[int]
    FLAT_FIELD_NUMBER: _ClassVar[int]
    IVF_FLAT_FIELD_NUMBER: _ClassVar[int]
    IVF_PQ_FIELD_NUMBER: _ClassVar[int]
    HNSW_FIELD_NUMBER: _ClassVar[int]
    DISKANN_FIELD_NUMBER: _ClassVar[int]
    USE_SCALAR_FILTER_FIELD_NUMBER: _ClassVar[int]
    VECTOR_FILTER_FIELD_NUMBER: _ClassVar[int]
    VECTOR_FILTER_TYPE_FIELD_NUMBER: _ClassVar[int]
    VECTOR_COPROCESSOR_FIELD_NUMBER: _ClassVar[int]
    VECTOR_IDS_FIELD_NUMBER: _ClassVar[int]
    USE_BRUTE_FORCE_FIELD_NUMBER: _ClassVar[int]
    LANGCHAIN_EXPR_FIELD_NUMBER: _ClassVar[int]
    top_n: int
    without_vector_data: bool
    without_scalar_data: bool
    selected_keys: _containers.RepeatedScalarFieldContainer[str]
    without_table_data: bool
    flat: SearchFlatParam
    ivf_flat: SearchIvfFlatParam
    ivf_pq: SearchIvfPqParam
    hnsw: SearchHNSWParam
    diskann: SearchDiskAnnParam
    use_scalar_filter: bool
    vector_filter: VectorFilter
    vector_filter_type: VectorFilterType
    vector_coprocessor: VectorCoprocessor
    vector_ids: _containers.RepeatedScalarFieldContainer[int]
    use_brute_force: bool
    langchain_expr: str
    def __init__(self, top_n: _Optional[int] = ..., without_vector_data: bool = ..., without_scalar_data: bool = ..., selected_keys: _Optional[_Iterable[str]] = ..., without_table_data: bool = ..., flat: _Optional[_Union[SearchFlatParam, _Mapping]] = ..., ivf_flat: _Optional[_Union[SearchIvfFlatParam, _Mapping]] = ..., ivf_pq: _Optional[_Union[SearchIvfPqParam, _Mapping]] = ..., hnsw: _Optional[_Union[SearchHNSWParam, _Mapping]] = ..., diskann: _Optional[_Union[SearchDiskAnnParam, _Mapping]] = ..., use_scalar_filter: bool = ..., vector_filter: _Optional[_Union[VectorFilter, str]] = ..., vector_filter_type: _Optional[_Union[VectorFilterType, str]] = ..., vector_coprocessor: _Optional[_Union[VectorCoprocessor, _Mapping]] = ..., vector_ids: _Optional[_Iterable[int]] = ..., use_brute_force: bool = ..., langchain_expr: _Optional[str] = ...) -> None: ...

class VectorCoprocessor(_message.Message):
    __slots__ = ["schema_version", "original_schema", "selection_columns", "expression"]
    SCHEMA_VERSION_FIELD_NUMBER: _ClassVar[int]
    ORIGINAL_SCHEMA_FIELD_NUMBER: _ClassVar[int]
    SELECTION_COLUMNS_FIELD_NUMBER: _ClassVar[int]
    EXPRESSION_FIELD_NUMBER: _ClassVar[int]
    schema_version: int
    original_schema: VectorSchemaWrapper
    selection_columns: _containers.RepeatedScalarFieldContainer[int]
    expression: bytes
    def __init__(self, schema_version: _Optional[int] = ..., original_schema: _Optional[_Union[VectorSchemaWrapper, _Mapping]] = ..., selection_columns: _Optional[_Iterable[int]] = ..., expression: _Optional[bytes] = ...) -> None: ...

class VectorSchemaWrapper(_message.Message):
    __slots__ = ["schema", "common_id"]
    SCHEMA_FIELD_NUMBER: _ClassVar[int]
    COMMON_ID_FIELD_NUMBER: _ClassVar[int]
    schema: _containers.RepeatedCompositeFieldContainer[ColumnDefinition]
    common_id: int
    def __init__(self, schema: _Optional[_Iterable[_Union[ColumnDefinition, _Mapping]]] = ..., common_id: _Optional[int] = ...) -> None: ...

class ScalarField(_message.Message):
    __slots__ = ["bool_data", "int_data", "long_data", "float_data", "double_data", "string_data", "bytes_data"]
    BOOL_DATA_FIELD_NUMBER: _ClassVar[int]
    INT_DATA_FIELD_NUMBER: _ClassVar[int]
    LONG_DATA_FIELD_NUMBER: _ClassVar[int]
    FLOAT_DATA_FIELD_NUMBER: _ClassVar[int]
    DOUBLE_DATA_FIELD_NUMBER: _ClassVar[int]
    STRING_DATA_FIELD_NUMBER: _ClassVar[int]
    BYTES_DATA_FIELD_NUMBER: _ClassVar[int]
    bool_data: bool
    int_data: int
    long_data: int
    float_data: float
    double_data: float
    string_data: str
    bytes_data: bytes
    def __init__(self, bool_data: bool = ..., int_data: _Optional[int] = ..., long_data: _Optional[int] = ..., float_data: _Optional[float] = ..., double_data: _Optional[float] = ..., string_data: _Optional[str] = ..., bytes_data: _Optional[bytes] = ...) -> None: ...

class ScalarValue(_message.Message):
    __slots__ = ["field_type", "fields"]
    FIELD_TYPE_FIELD_NUMBER: _ClassVar[int]
    FIELDS_FIELD_NUMBER: _ClassVar[int]
    field_type: ScalarFieldType
    fields: _containers.RepeatedCompositeFieldContainer[ScalarField]
    def __init__(self, field_type: _Optional[_Union[ScalarFieldType, str]] = ..., fields: _Optional[_Iterable[_Union[ScalarField, _Mapping]]] = ...) -> None: ...

class VectorIndexMetrics(_message.Message):
    __slots__ = ["vector_index_type", "current_count", "deleted_count", "max_id", "min_id", "memory_bytes"]
    VECTOR_INDEX_TYPE_FIELD_NUMBER: _ClassVar[int]
    CURRENT_COUNT_FIELD_NUMBER: _ClassVar[int]
    DELETED_COUNT_FIELD_NUMBER: _ClassVar[int]
    MAX_ID_FIELD_NUMBER: _ClassVar[int]
    MIN_ID_FIELD_NUMBER: _ClassVar[int]
    MEMORY_BYTES_FIELD_NUMBER: _ClassVar[int]
    vector_index_type: VectorIndexType
    current_count: int
    deleted_count: int
    max_id: int
    min_id: int
    memory_bytes: int
    def __init__(self, vector_index_type: _Optional[_Union[VectorIndexType, str]] = ..., current_count: _Optional[int] = ..., deleted_count: _Optional[int] = ..., max_id: _Optional[int] = ..., min_id: _Optional[int] = ..., memory_bytes: _Optional[int] = ...) -> None: ...



"""Internal representation of a structured query language."""
from __future__ import annotations

from abc import ABC, abstractmethod
from enum import Enum
from typing import Any, List, Optional, Sequence, Union

from pydantic import BaseModel


class Visitor(ABC):
    """Defines interface for IR translation using visitor pattern."""

    allowed_comparators: Optional[Sequence[Comparator]] = None
    allowed_operators: Optional[Sequence[Operator]] = None

    def _validate_func(self, func: Union[Operator, Comparator]) -> None:
        if isinstance(func, Operator) and self.allowed_operators is not None:
            if func not in self.allowed_operators:
                raise ValueError(
                    f"Received disallowed operator {func}. Allowed "
                    f"comparators are {self.allowed_operators}"
                )
        if isinstance(func, Comparator) and self.allowed_comparators is not None:
            if func not in self.allowed_comparators:
                raise ValueError(
                    f"Received disallowed comparator {func}. Allowed "
                    f"comparators are {self.allowed_comparators}"
                )

    @abstractmethod
    def visit_operation(self, operation: Operation) -> Any:
        """Translate an Operation."""

    @abstractmethod
    def visit_comparison(self, comparison: Comparison) -> Any:
        """Translate a Comparison."""

    @abstractmethod
    def visit_structured_query(self, structured_query: StructuredQuery) -> Any:
        """Translate a StructuredQuery."""


def _to_snake_case(name: str) -> str:
    """Convert a name into snake_case."""
    snake_case = ""
    for i, char in enumerate(name):
        if char.isupper() and i != 0:
            snake_case += "_" + char.lower()
        else:
            snake_case += char.lower()
    return snake_case


class Expr(BaseModel):
    """Base class for all expressions."""

    def accept(self, visitor: Visitor) -> Any:
        """Accept a visitor.

        Args:
            visitor: visitor to accept

        Returns:
            result of visiting
        """
        return getattr(visitor, f"visit_{_to_snake_case(self.__class__.__name__)}")(
            self
        )


class Operator(str, Enum):
    """Enumerator of the operations."""

    AND = "and"
    OR = "or"
    NOT = "not"


class Comparator(str, Enum):
    """Enumerator of the comparison operators."""

    EQ = "eq"
    GT = "gt"
    GTE = "gte"
    LT = "lt"
    LTE = "lte"
    CONTAIN = "contain"
    LIKE = "like"


class FilterDirective(Expr, ABC):
    """A filtering expression."""


class Comparison(FilterDirective):
    """A comparison to a value."""

    comparator: Comparator
    attribute: str
    value: Any


class Operation(FilterDirective):
    """A logical operation over other directives."""

    operator: Operator
    arguments: List[FilterDirective]


class StructuredQuery(Expr):
    """A structured query."""

    query: str
    """Query string."""
    filter: Optional[FilterDirective]
    """Filtering expression."""
    limit: Optional[int]
    """Limit on the number of results."""



import warnings
from typing import List, Union

from pydantic import BaseModel, validator

from . import config
from dingodb.utils.tools import auto_value_type, auto_expr_type, convert_dict_to_expr


class CheckClintParam(BaseModel):
    user: str = "root"
    password: str = "123123"
    host: List[str]


class CheckCreateIndexParam(BaseModel):
    index_name: str
    dimension: int
    index_type: str = "hnsw"
    metric_type: str = "euclidean"
    replicas: int = 3
    index_config: dict = None
    metadata_config: dict = None
    partition_rule: dict = None
    operand: list = None
    auto_id: bool = True
    start_id: int = 1

    @validator("index_type", always=True)
    def check_index_type(cls, value):
        if value not in config.index_config.keys():
            raise Exception(f"index_type  must in {list(config.index_config.keys())}")
        return value

    @validator("metric_type", always=True)
    def check_metric_type(cls, value):
        if value not in config.metric_type.keys():
            raise Exception(f"metric_type  must in {list(config.metric_type.keys())}")
        value = config.metric_type[value]
        return value

    @validator("replicas", always=True)
    def check_replicas(cls, value):
        if value < 0:
            raise Exception(f"{value} must >= 0")
        return value

    @validator("index_config", always=True)
    def check_index_config(cls, value, values):
        index_type = values.get("index_type")
        dimension = values.get("dimension")
        metric_type = values.get("metric_type")
        vector_index_parameter = config.index_config[index_type]

        if value is not None:
            index_keys = vector_index_parameter[
                list(vector_index_parameter.keys())[0]
            ].keys()
            for key, v in value.items():
                if key in index_keys:
                    vector_index_parameter[list(vector_index_parameter.keys())[0]][
                        key
                    ] = v
                    if key == "efConstruction":
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        if v > 500 or v < 100:
                            warnings.warn(f"efConstruction: {v} suggestion in 100-500")
                    if key == "maxElements":
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        if v > 1000000000 or v < 50000:
                            warnings.warn(
                                f"maxElements:{v} suggestion in 50000-1000000000"
                            )
                    if key == "nlinks":
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        if v > 64 or v < 16:
                            warnings.warn(f"nlinks:{v} suggestion in 16-64")

                    if key == "nsubvector":
                        if dimension % v != 0:
                            raise Exception(f"dimension/nsubvector must Divisible")
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                        if v > 256 or v < 4:
                            warnings.warn(f"nsubvector:{v} suggestion in 4-256")
                    if key == "ncentroids":
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                        if v > 4096 or v < 4:
                            warnings.warn(f"ncentroids:{v} suggestion in 4-4096")
                    if key == "bucketInitSize":
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                    if key == "bucketMaxSize":
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                    if key == "nbitsPerIdx":
                        if v <= 0 or v > 16:
                            raise Exception(f"{key} must > 0 and <=16")
                else:
                    warnings.warn(f"index_config {key} not in {index_type}")

        vector_index_parameter[list(vector_index_parameter.keys())[0]][
            "dimension"
        ] = dimension
        vector_index_parameter[list(vector_index_parameter.keys())[0]][
            "metricType"
        ] = metric_type
        return vector_index_parameter

    @validator("partition_rule", always=True)
    def check_partition_rule(cls, value):
        value = {} if value is None else value
        return value


class CheckVectorAddParam(BaseModel):
    index_name: str
    vectors: List[list]
    datas: List[dict] = None
    ids: List[int] = None

    @validator("datas", always=True)
    def check_datas(cls, value, values):
        value = [{}] * len(values.get("vectors")) if value is None else value
        return value

    @validator("ids", always=True)
    def check_ids(cls, value, values):
        if value is None:
            assert len(values.get("datas")) == len(
                values.get("vectors")
            ), f"length datas:{len(values.get('datas'))} vectors: {len(values.get('vectors'))} is not equal"
        else:
            assert (
                len(values.get("datas")) == len(values.get("vectors")) == len(value)
            ), f"length datas:{len(values.get('datas'))} vectors: {len(values.get('vectors'))} ids:{len(value)} is not equal"
            for id in value:
                if id <= 0:
                    raise Exception("id must > 0")
        return value


class CheckVectorScanParam(BaseModel):
    index_name: str
    start_id: int
    max_count: int = 1000
    is_reverse: bool = False
    without_scalar_data: bool = False
    without_table_data: bool = False
    without_vector_data: bool = False
    fields: list = None
    filter_scalar: dict = None
    end_id: int = 0

    @validator("end_id", pre=True, always=True)
    def check_true_positive_values(cls, value, field):
        if not value >= 0:
            raise ValueError(f"{field.name} must >= 0")
        return value

    @validator("start_id", "max_count", pre=True, always=True)
    def check_positive_values(cls, value, field):
        if not value > 0:
            raise ValueError(f"{field.name} must > 0")
        return value

    @validator(
        "is_reverse", "without_scalar_data", "without_table_data", pre=True, always=True
    )
    def check_boolean_fields(cls, value):
        return "true" if value else "false"

    @validator("without_vector_data", pre=True, always=True)
    def check_without_vector_data(cls, value):
        return "true" if value else "false"

    @validator("fields", pre=True, always=True)
    def check_fields(cls, value):
        return value or []

    @validator("filter_scalar", pre=True, always=True)
    def check_filter_scalar(cls, value):
        if value is None:
            return {}
        else:
            scalar_data = dict(
                (
                    key,
                    {"fieldType": auto_value_type(value), "fields": [{"data": value}]},
                )
                for key, value in value.items()
            )
            value = scalar_data
            return value


class CheckVectorSearchParam(BaseModel):
    index_name: str
    xq: list
    index_type: str
    top_k: int = 10
    pre_filter: bool = True
    brute: bool = False
    search_params: dict = None

    @validator("xq", always=True)
    def check_xq(cls, value):
        if not isinstance(value[0], list):
            value = [value]
        return value

    @validator("top_k", pre=True, always=True)
    def check_top_k(cls, value, field):
        if not value > 0:
            raise ValueError(f"{field.name} must > 0")
        return value

    @validator("search_params", always=True)
    def check_search_params(cls, search_params, values):
        scalar_data = {}
        use_scalar_filter = "false"
        lanchain_json = None
        if search_params is not None:
            if (
                "meta_expr" in search_params.keys()
                and "langchain_expr" in search_params.keys()
            ):
                raise ValueError(f"meta_expr and langchain_expr cannot coexist")
            elif "meta_expr" in search_params.keys():
                if search_params["meta_expr"] is not None:
                    lanchain_json = auto_expr_type(
                        convert_dict_to_expr(search_params["meta_expr"])
                    )

            elif "langchain_expr" in search_params.keys():
                if search_params["langchain_expr"] is not None:
                    lanchain_json = auto_expr_type(search_params["langchain_expr"])
            else:
                values["pre_filter"] = False
        else:
            values["pre_filter"] = False

        ef_search = 32 if search_params is None else search_params.get("efSearch", 32)
        assert ef_search >= 0, f"efSearch must >= 0, but get {ef_search}"

        nprobe = 16 if search_params is None else search_params.get("nprobe", 16)
        assert nprobe >= 0, f"nprobe must >= 0, but get {nprobe}"
        recallNum = (
            100 if search_params is None else search_params.get("recallNum", 100)
        )
        assert recallNum > 0, f"recallNum must > 0, but get {recallNum}"
        index_type = values.get("index_type")

        with_Scalar_data = "false"
        with_vector_data = "false"
        if search_params is not None:
            if search_params.get("withScalarData") is not None:
                with_Scalar_data = search_params.get("withScalarData")
                with_Scalar_data = (
                    "false" if with_Scalar_data is None else with_Scalar_data
                )

            if search_params.get("withVectorData") is not None:
                with_vector_data = search_params.get("withVectorData")
                with_vector_data = (
                    "false" if with_vector_data is None else with_vector_data
                )
        parallel = (
            0 if search_params is None else search_params.get("parallelOnQueries", 0)
        )
        if index_type == "VECTOR_INDEX_TYPE_HNSW":
            search = {"hnswParam": {"efSearch": ef_search}}
        elif index_type == "VECTOR_INDEX_TYPE_FLAT":
            search = {"flat": {"parallelOnQueries": parallel}}
        elif index_type == "VECTOR_INDEX_TYPE_BRUTEFORCE":
            search = {}
        elif index_type == "VECTOR_INDEX_TYPE_IVF_FLAT":
            search = {"ivfFlatParam": {"nprobe": nprobe, "parallelOnQueries": parallel}}
        elif index_type == "VECTOR_INDEX_TYPE_IVF_PQ":
            search = {
                "ivfPqParam": {
                    "nprobe": nprobe,
                    "parallelOnQueries": parallel,
                    "recallNum": recallNum,
                }
            }

        payload = {
            "parameter": {
                "useBruteForce": "true" if values.get("brute") else "false",
                "search": search,
                "selectedKeys": [],
                "topN": values.get("top_k"),
                "withoutScalarData": with_Scalar_data,
                "withoutVectorData": with_vector_data,
                "useScalarFilter": use_scalar_filter,
                "vectorFilter": "SCALAR_FILTER",
                "vectorFilterType": "QUERY_PRE"
                if values.get("pre_filter")
                else "QUERY_POST",
            },
            "vectors": [
                {
                    "scalarData": scalar_data,
                    "vector": {
                        "binaryValues": [],
                        "dimension": len(xq),
                        "floatValues": xq,
                        "valueType": "FLOAT",
                    },
                }
                for xq in values.get("xq")
            ],
        }
        payload["parameter"].update(
            {
                "langchainExpr": lanchain_json,
            }
        ) if lanchain_json is not None else ...
        return payload


class CheckVectorGetParam(BaseModel):
    index_name: str
    ids: List[int]
    scalar: bool = True
    vector: bool = True

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        assert len(id_list) > 0, f"ids list length must > 0, but get {id_list}"
        return id_list


class CheckVectorDeleteParam(BaseModel):
    index_name: str
    ids: List[int]

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        assert len(id_list) > 0, f"ids list length must > 0, but get {id_list}"
        return id_list



from typing import List, Dict, Union

from dingodb.sdk_client import SDKClient
from dingodb.sdk_document.sdk_document_client import SDKDocumentClient

from dingodb.common.document_rep import (
    DocumentSchema,
    AddResult,
    DocSearchResult,
    DocQueryResult,
    DocScanQueryResult,
    DocIndexMetricsResult,
    DocDeleteResult,
)

from .sdk_document_param import (
    CreateIndexParam,
    DocumentDeleteParam,
    DocumentAddParam,
    DocumentSearchParam,
    DocumentQueryParam,
    DocumentGetBorderParam,
    DocumentScanQueryParam,
    DocumentCountParam,
)


class SDKDocumentDingoDB:
    def __init__(self, client: SDKClient):
        """
        __init__ init DingoSDK

        Args:
            client: SDKClient
        """
        self.client = SDKDocumentClient(client)

    def create_index(
        self,
        index_name: str,
        schema: DocumentSchema,
        replicas: int = 3,
        operand: List[int] = None,
        start_id: int = 0,
        json_params: str = "",
    ) -> bool:
        """
        create_index create index

        Args:
            index_name (str): the name of index
            schema (DocumentSchema): the schema of index
            replicas (int, optional): dingoDB store replicas. Defaults to 3.
            operand (list, optional): operand. Defaults to None.
            start_id (int, optional): autoIncrement start id. Defaults to 0.
            json_params (str, optional): the json params of index. Defaults to "".

        Raises:
            RuntimeError: return error

        Returns:
            bool: create table status
        """
        params = CreateIndexParam(
            index_name=index_name,
            replicas=replicas,
            operand=operand,
            start_id=start_id,
            json_params=json_params,
        )

        return self.client.create_index(params, schema=schema)

    def delete_index(self, index_name: str) -> bool:
        """
        delete_index del/drop index

        Args:
            index_name (str): the name of index

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """
        return self.client.delete_index(index_name)

    def get_schema(self, index_name: str) -> DocumentSchema:
        """
        get_schema get dschema

        Args:
            index_name (str): the name of index

        Raises:
            RuntimeError: return error

        Returns:
            DocumentSchema: dingodb.common.document_rep.DocumentSchema
        """

        return self.client.get_schema(index_name)

    def document_add(
        self, index_name: str, documents: List, ids: List[int] = None
    ) -> AddResult:
        """
        document_add add document

        Args:
            index_name (str): the name of index
            documents (list): documents list
            ids (list): id list

        Raises:
            RuntimeError: return error

        Returns:
            AddResult: dingodb.common.document_rep.AddResult
        """
        params = DocumentAddParam(index_name=index_name, documents=documents, ids=ids)

        return self.client.document_add(params)

    def document_search(
        self,
        index_name: str,
        query_string: str,
        top_n: int,
        doc_ids: List = None,
        with_scalar_data: bool = False,
        column_names: List[str] = None,
        selected_keys: List[str] = None,
    ) -> DocSearchResult:
        """
        document_search search document

        Args:
            index_name (str): the name of the index
            query_string (str): the query string
            top_n (int, optional): top n search. Defaults to 0.
            doc_ids (list, optional): document ids. Defaults to None.
            with_scalar_data (bool, optional): whether to include scalar data. Defaults to True.
            column_names (list, optional): column names. Defaults to None.
            selected_keys (list, optional): selected keys. Defaults to None.

        Raises:
            RuntimeError: return error

        Returns:
            DocSearchResult: dingodb.common.document_rep.DocSearchResult
        """

        params = DocumentSearchParam(
            index_name=index_name,
            query_string=query_string,
            top_n=top_n,
            doc_ids=doc_ids,
            with_scalar_data=with_scalar_data,
            column_names=column_names,
            selected_keys=selected_keys,
        )

        return self.client.document_search(params)

    def document_query(
        self,
        index_name: str,
        doc_ids: List,
        with_scalar_data: bool = False,
        selected_keys: List = None,
    ) -> DocQueryResult:
        """
        document_query query document

        Args:
            index_name (str): the name of the index
            doc_ids (list): document ids.
            with_scalar_data (bool, optional): whether to include scalar data. Defaults to False. if true, response with scalar data
            selected_keys (list, optional): selected keys. Defaults to None.
                If with_scalar_data is true, selected_keys is used to select scalar data, and if this parameter is null, all scalar data will be returned.

        Raises:
            RuntimeError: return error

        Returns:
            DocQueryResult: dingodb.common.document_rep.DocQueryResult
        """

        params = DocumentQueryParam(
            index_name=index_name,
            doc_ids=doc_ids,
            with_scalar_data=with_scalar_data,
            selected_keys=selected_keys,
        )

        return self.client.document_query(params)

    def document_get_border(
        self,
        index_name: str,
        is_max: bool = True,
    ) -> int:
        """
        document_get_border get_border document

        Args:
            index_name (str): the name of the index
            is_max (bool, optional): whether to return max. Defaults to True.

        Raises:
            RuntimeError: return error

        Returns:
            int: get_border results
        """

        params = DocumentGetBorderParam(index_name=index_name, is_max=is_max)

        return self.client.document_get_border(params)

    def document_scan_query(
        self,
        index_name: str,
        doc_id_start: int,
        doc_id_end: int = 0,
        is_reverse: bool = False,
        max_scan_count: int = 1000,
        with_scalar_data: bool = True,
        selected_keys: List[str] = None,
    ) -> DocScanQueryResult:
        """
        document_scan_query scan_query document

        Args:
            index_name (str): the name of the index
            doc_id_start (int): start document id
            doc_id_end (int): end document id, the end id of scan
                if is_reverse is true, doc_id_end must be less than doc_id_start
                if is_reverse is false, doc_id_end must be greater than doc_id_start
                the real range is [start, end], include start and end
                if doc_id_end == 0, scan to the end of the region
            is_reverse is true, doc_id_end must be greater than doc_id_start
            max_scan_count (int, optional): maximum number of scans. Defaults to 1000.
            with_scalar_data (bool, optional): whether to include scalar data. Defaults to False.
            selected_keys (list, optional): selected keys. Defaults to None.
                If with_scalar_data is true, selected_keys is used to select scalar data, and if this parameter is null, all scalar data will be returned.

        Raises:
            RuntimeError: return error

        Returns:
            DocScanQueryResult: dingodb.common.document_rep.DocScanQueryResult
        """

        params = DocumentScanQueryParam(
            index_name=index_name,
            doc_id_start=doc_id_start,
            doc_id_end=doc_id_end,
            is_reverse=is_reverse,
            max_scan_count=max_scan_count,
            with_scalar_data=with_scalar_data,
            selected_keys=selected_keys,
        )

        return self.client.document_scan_query(params)

    def document_index_metrics(self, index_name: str) -> DocIndexMetricsResult:
        """
        document_index_metrics index_metrics in index

        Args:
            index_name (str): the name of in index

        Raises:
            RuntimeError: return error

        Returns:
            DocIndexMetricsResult: dingodb.common.document_rep.DocIndexMetricsResult
        """
        return self.client.document_index_metrics(index_name)

    def document_count(
        self,
        index_name: str,
        doc_id_start: int,
        doc_id_end: int,
    ) -> int:
        """
        document_count count in index

        Args:
            index_name (str): the name of in index
            doc_id_start (int): start document id
            doc_id_end (int): end document id, the end id of scan

        Raises:
            RuntimeError: return error

        Returns:
            int: count int
        """

        params = DocumentCountParam(
            index_name=index_name, doc_id_start=doc_id_start, doc_id_end=doc_id_end
        )

        return self.client.document_count(params)

    def document_delete(self, index_name: str, ids: List) -> List[bool]:
        """
        document_delete delete document with ids

        Args:
            index_name (str): the name of the index
            ids (list): id list

        Raises:
            RuntimeError: return error

        Returns:
            List[bool]: [True, False, ...]
        """
        params = DocumentDeleteParam(index_name=index_name, ids=ids)
        document_del_map = {
            d.doc_id: d.deleted for d in self.client.document_delete(params)
        }

        return [document_del_map.get(i) for i in params.ids]

    def document_get_auto_increment_id(self, index_name: str) -> int:
        """
        document_get_auto_increment_id get_auto_increment_id

        Args:
           index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            auto_increment_id
        """
        return self.client.document_get_auto_increment_memory(index_name=index_name)

    def document_update_auto_increment_id(self, index_name: str, start_id: int):
        """
        document_update_auto_increment_id update AutoIncrementId

        Args:
           index_name: str

        Raises:
            RuntimeError: return error

        Returns:

        """
        return self.client.document_update_auto_increment_id(
            index_name=index_name, start_id=start_id
        )



#!/usr/bin/python3

import json
import dingosdk
from dingodb.sdk_client import SDKClient

from dingodb.utils.tools import auto_value_type
from dingodb.common import constants
from dingodb.common.vector_rep import (
    ScalarSchema,
    ScalarType,
    RegionState,
    RegionStatus,
)
from dingodb.utils.tools import auto_value_type, auto_expr_type, convert_dict_to_expr

from .sdk_vector_param import (
    CreateIndexParam,
    VectorAddParam,
    VectorScanParam,
    VectorSearchParam,
    VectorGetParam,
    VectorDeleteParam,
    VectorStatusByRegionIdParam,
    VectorBuildByRegionIdParam,
    VectorLoadByRegionIdParam,
    VectorResetByRegionIdParam,
)

from .sdk_vector_param_factory import SDKParamFactory
from .sdk_vector_adapter import (
    sdk_search_result_to_search_result,
    sdk_vector_with_id_to_vector_with_id,
    sdk_index_metrics_result_to_index_metric,
    sdk_err_status_result_to_err_status,
    sdk_state_result_to_state,
    type_conversion,
)

sdk_types = {
    "STRING": dingosdk.Type.kSTRING,
    "DOUBLE": dingosdk.Type.kDOUBLE,
    "INT64": dingosdk.Type.kINT64,
    "BOOL": dingosdk.Type.kBOOL,
}

scalar_type_to_sdk_type = {
    ScalarType.BOOL: dingosdk.Type.kBOOL,
    ScalarType.INT64: dingosdk.Type.kINT64,
    ScalarType.DOUBLE: dingosdk.Type.kDOUBLE,
    ScalarType.STRING: dingosdk.Type.kSTRING,
}


class SDKVectorClient:
    def __init__(self, client: SDKClient):
        """
        __init__ init Client

        Args:
            client: SDKClient
        """
        self.schema_id = 2
        self.client = client.dingosdk_client

        s, self.vector_client = self.client.NewVectorClient()
        if not s.ok():
            raise RuntimeError(f"dongo vector client build fail: {s.ToString()}")

    def create_index(
        self, param: CreateIndexParam, schema: ScalarSchema = None
    ) -> bool:
        """
        create_index create index

        Args:
            param (CreateIndexParam): create index param
            schema (ScalarSchema): scalar data schema

        Returns:
            bool: create index result
        """
        s, creator = self.client.NewVectorIndexCreator()
        assert s.ok(), f"dingo creator build fail: {s.ToString()}"
        creator.SetSchemaId(self.schema_id)
        creator.SetName(param.index_name)
        creator.SetReplicaNum(param.replicas)
        if param.operand is not None:
            if len(param.operand) != 0:
                creator.SetRangePartitions(param.operand)

        index_type = param.index_type
        if index_type == "flat":
            sdk_param = SDKParamFactory.create_flat_param(
                index_type, param.index_config
            )
            creator.SetFlatParam(sdk_param)
        elif index_type == "ivf_flat":
            sdk_param = SDKParamFactory.create_ivf_flat_param(
                index_type, param.index_config
            )
            creator.SetIvfFlatParam(sdk_param)
        elif index_type == "ivf_pq":
            sdk_param = SDKParamFactory.create_ivf_pq_param(
                index_type, param.index_config
            )
            creator.SetIvfPqParam(sdk_param)
        elif index_type == "hnsw":
            sdk_param = SDKParamFactory.create_hnsw_param(
                index_type, param.index_config
            )
            creator.SetHnswParam(sdk_param)
        elif index_type == "diskann":
            sdk_param = SDKParamFactory.create_diskann_param(
                index_type, param.index_config
            )
            creator.SetDiskAnnParam(sdk_param)
        elif index_type == "brute":
            sdk_param = SDKParamFactory.create_brute_param(
                index_type, param.index_config
            )
            creator.SetBruteForceParam(sdk_param)
        elif index_type == "binary_flay":
            sdk_param = SDKParamFactory.create_binary_flat_param(
                index_type,param.index_config
            )
            creator.SetBinaryFlatParam(sdk_param)
        elif index_type == "binary_ivf_flat":
            sdk_param = SDKParamFactory.create_binary_ivf_flat_param(
                index_type,param.index_config
            )
            creator.SetBinaryIvfFlatParam(sdk_param)
        else:
            raise RuntimeError(f"index_type: {index_type} not support")

        if param.auto_id:
            creator.SetAutoIncrementStart(param.start_id)

        if schema is not None and len(schema.cols) != 0:
            sdk_scalar_schem = dingosdk.VectorScalarSchema()

            for col in schema.cols:
                sdk_col = dingosdk.VectorScalarColumnSchema(
                    col.key, scalar_type_to_sdk_type[col.type], col.speed
                )

                sdk_scalar_schem.AddScalarColumn(sdk_col)

            creator.SetScalarSchema(sdk_scalar_schem)

        # index_id = -1
        s, index_id = creator.Create()
        if s.ok():
            return True
        else:
            raise RuntimeError(
                f"create index {param.index_name} fail: {s.ToString()}, index_id: {index_id}"
            )

    def delete_index(self, index_name: str) -> bool:
        """
        delete_index del/drop index

        Args:
            index_name (str): the name of index

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """
        s = self.client.DropVectorIndexByName(self.schema_id, index_name)
        if s.ok():
            return True
        else:
            raise RuntimeError(f"delete index {index_name} fail: {s.ToString()}")

    def vectors_add_with_schema(
        self, schema_dict: dict, add_param: VectorAddParam
    ) -> list:
        vectors = []
        for i, v in enumerate(add_param.vectors):
            id = 0
            if add_param.ids is not None:
                id = add_param.ids[i]

            if add_param.value_type == "float" :
                tmp_vector = dingosdk.Vector(dingosdk.ValueType.kFloat, len(v))
                tmp_vector.float_values = v
            elif add_param.value_type=="binary" :
                tmp_vector = dingosdk.Vector(dingosdk.ValueType.kUint8, len(v))
                tmp_vector.binary_values = v

            vector_with_id = dingosdk.VectorWithId(id, tmp_vector)

            scarlar_data = {}
            for key, value in add_param.datas[i].items():

                scalar_type = sdk_types[auto_value_type(value)]

                if key in schema_dict:
                    if not type_conversion(scalar_type, schema_dict[key]):
                        raise RuntimeError(
                            f"type_conversion error {scalar_type} to {schema_dict[key]}"
                        )
                    scalar_type = schema_dict[key]

                scarlar_value = dingosdk.ScalarValue()
                scarlar_field = dingosdk.ScalarField()

                if scalar_type == dingosdk.Type.kSTRING:
                    scarlar_value.type = dingosdk.Type.kSTRING
                    scarlar_field.string_data = value
                elif scalar_type == dingosdk.Type.kDOUBLE:
                    scarlar_value.type = dingosdk.Type.kDOUBLE
                    scarlar_field.double_data = value
                elif scalar_type == dingosdk.Type.kINT64:
                    scarlar_value.type = dingosdk.Type.kINT64
                    scarlar_field.long_data = value
                elif scalar_type == dingosdk.Type.kBOOL:
                    scarlar_value.type = dingosdk.Type.kBOOL
                    scarlar_field.bool_data = value
                else:
                    raise RuntimeError(f"not support type: {scalar_type}")

                # TODO: support vector with multiple fields
                fields = []
                fields.append(scarlar_field)
                scarlar_value.fields = fields

                scarlar_data[key] = scarlar_value

            vector_with_id.scalar_data = scarlar_data
            vectors.append(vector_with_id)
        return vectors

    def vectors_add_without_schema(self, add_param: VectorAddParam) -> list:
        vectors = []
        for i, v in enumerate(add_param.vectors):
            id = 0
            if add_param.ids is not None:
                id = add_param.ids[i]

            if add_param.value_type == "float" :
                tmp_vector = dingosdk.Vector(dingosdk.ValueType.kFloat, len(v))
                tmp_vector.float_values = v
            elif add_param.value_type=="binary" :
                tmp_vector = dingosdk.Vector(dingosdk.ValueType.kUint8, len(v))
                tmp_vector.binary_values = v


            vector_with_id = dingosdk.VectorWithId(id, tmp_vector)

            scarlar_data = {}
            for key, value in add_param.datas[i].items():
                scarlar_value = dingosdk.ScalarValue()
                scarlar_value.type = sdk_types[auto_value_type(value)]

                scalar_type = scarlar_value.type
                scarlar_field = dingosdk.ScalarField()
                if scalar_type == dingosdk.Type.kSTRING:
                    scarlar_field.string_data = value
                elif scalar_type == dingosdk.Type.kDOUBLE:
                    scarlar_field.double_data = value
                elif scalar_type == dingosdk.Type.kINT64:
                    scarlar_field.long_data = value
                elif scalar_type == dingosdk.Type.kBOOL:
                    scarlar_field.bool_data = value
                else:
                    raise RuntimeError(f"not support type: {scarlar_value.type}")

                # TODO: support vector with multiple fields
                fields = []
                fields.append(scarlar_field)
                scarlar_value.fields = fields

                scarlar_data[key] = scarlar_value

            vector_with_id.scalar_data = scarlar_data
            vectors.append(vector_with_id)
        return vectors

    def vector_add(self, add_param: VectorAddParam) -> list:
        """
        vector_add add vector

        Args:
            add_param: VectorAddParam

        Raises:
            RuntimeError: return error

        Returns:
            list: vector id list
        """

        status, out_vector_index = self.client.GetVectorIndex(
            self.schema_id, add_param.index_name
        )
        if not status.ok():
            raise RuntimeError(
                f"get index {add_param.index_name} error: {status.ToString()}"
            )
        else:
            if out_vector_index.HasScalarSchema():
                schema_dict = out_vector_index.GetSchema()
                vectors = self.vectors_add_with_schema(schema_dict, add_param)
            else:
                vectors = self.vectors_add_without_schema(add_param)

        s, vectors = self.vector_client.AddByIndexName(
            self.schema_id, add_param.index_name, vectors
        )

        if s.ok():
            return [sdk_vector_with_id_to_vector_with_id(v).to_dict() for v in vectors]
        else:
            raise RuntimeError(
                f"add vector in {add_param.index_name} fail: {s.ToString()}"
            )

    def vector_upsert(self, upsert_param: VectorAddParam) -> list:
        """
        vector_upsert upsert vector

        Args:
            upsert_param: VectorAddParam

        Raises:
            RuntimeError: return error

        Returns:
            list: vector id list
        """

        status, out_vector_index = self.client.GetVectorIndex(
            self.schema_id, upsert_param.index_name
        )
        if not status.ok():
            raise RuntimeError(
                f"get index {upsert_param.index_name} error: {status.ToString()}"
            )
        else:
            if out_vector_index.HasScalarSchema():
                schema_dict = out_vector_index.GetSchema()
                vectors = self.vectors_add_with_schema(schema_dict, upsert_param)
            else:
                vectors = self.vectors_add_without_schema(upsert_param)

        s, vectors = self.vector_client.UpsertByIndexName(
            self.schema_id, upsert_param.index_name, vectors
        )

        if s.ok():
            return [sdk_vector_with_id_to_vector_with_id(v).to_dict() for v in vectors]
        else:
            raise RuntimeError(
                f"upsert vector in {upsert_param.index_name} fail: {s.ToString()}"
            )

    def vector_count(self, index_name: str):
        """
        vector_count count in index

        Args:
            index_name (str): the name of in index

        Raises:
            RuntimeError: return error

        Returns:
            int: count num
        """
        s, count = self.vector_client.CountallByIndexName(self.schema_id, index_name)
        if s.ok():
            return count
        else:
            raise RuntimeError(f"count index {index_name} fail: {s.ToString()}")

    def vector_metrics(self, index_name: str):
        """
        vector_metrics metrics in index

        Args:
            index_name (str): the name of in index

        Returns:
            str: metrics string
        """
        s, result = self.vector_client.GetIndexMetricsByIndexName(
            self.schema_id, index_name
        )
        if s.ok():
            return sdk_index_metrics_result_to_index_metric(result).to_dict()
        else:
            raise RuntimeError(f"get index {index_name} metrics fail: {s.ToString()}")

    def vectors_scan_with_schema(self, schema_dict: dict, scan_param: VectorScanParam):
        sdk_param = dingosdk.ScanQueryParam()
        sdk_param.vector_id_start = scan_param.start_id
        sdk_param.vector_id_end = scan_param.end_id
        sdk_param.max_scan_count = scan_param.max_count
        sdk_param.is_reverse = scan_param.is_reverse
        sdk_param.with_vector_data = scan_param.with_vector_data
        sdk_param.with_scalar_data = scan_param.with_scalar_data
        sdk_param.with_table_data = scan_param.with_table_data
        selected_keys = []
        for key in scan_param.fields:
            selected_keys.append(key)
        sdk_param.selected_keys = selected_keys

        if scan_param.filter_scalar:
            sdk_param.use_scalar_filter = True
            scarlar_data = {}
            for key, value in scan_param.filter_scalar.items():

                scalar_type = sdk_types[auto_value_type(value)]

                if key in schema_dict:
                    if not type_conversion(scalar_type, schema_dict[key]):
                        raise RuntimeError(
                            f"type_conversion error {scalar_type} to {schema_dict[key]}"
                        )
                    scalar_type = schema_dict[key]

                scarlar_value = dingosdk.ScalarValue()
                scarlar_field = dingosdk.ScalarField()

                if scalar_type == dingosdk.Type.kSTRING:
                    scarlar_value.type = dingosdk.Type.kSTRING
                    scarlar_field.string_data = value
                elif scalar_type == dingosdk.Type.kDOUBLE:
                    scarlar_value.type = dingosdk.Type.kDOUBLE
                    scarlar_field.double_data = value
                elif scalar_type == dingosdk.Type.kINT64:
                    scarlar_value.type = dingosdk.Type.kINT64
                    scarlar_field.long_data = value
                elif scalar_type == dingosdk.Type.kBOOL:
                    scarlar_value.type = dingosdk.Type.kBOOL
                    scarlar_field.bool_data = value
                else:
                    raise RuntimeError(f"not support type: {scalar_type}")

                # TODO: support vector with multiple fields
                fields = []
                fields.append(scarlar_field)
                scarlar_value.fields = fields

                scarlar_data[key] = scarlar_value

            sdk_param.scalar_data = scarlar_data

        return sdk_param

    def vectors_scan_without_schema(self, scan_param: VectorScanParam):
        sdk_param = dingosdk.ScanQueryParam()
        sdk_param.vector_id_start = scan_param.start_id
        sdk_param.vector_id_end = scan_param.end_id
        sdk_param.max_scan_count = scan_param.max_count
        sdk_param.is_reverse = scan_param.is_reverse
        sdk_param.with_vector_data = scan_param.with_vector_data
        sdk_param.with_scalar_data = scan_param.with_scalar_data
        sdk_param.with_table_data = scan_param.with_table_data
        selected_keys = []
        for key in scan_param.fields:
            selected_keys.append(key)
        sdk_param.selected_keys = selected_keys

        if scan_param.filter_scalar:
            sdk_param.use_scalar_filter = True
            scarlar_data = {}
            for key, value in scan_param.filter_scalar.items():
                scarlar_value = dingosdk.ScalarValue()
                scarlar_value.type = sdk_types[auto_value_type(value)]
                scalar_type = scarlar_value.type

                scarlar_field = dingosdk.ScalarField()
                if scalar_type == dingosdk.Type.kSTRING:
                    scarlar_field.string_data = value
                elif scalar_type == dingosdk.Type.kDOUBLE:
                    scarlar_field.double_data = value
                elif scalar_type == dingosdk.Type.kINT64:
                    scarlar_field.long_data = value
                elif scalar_type == dingosdk.Type.kBOOL:
                    scarlar_field.bool_data = value
                else:
                    raise RuntimeError(f"not support type: {scarlar_value.type}")

                # TODO: support vector with multiple fields
                fields = []
                fields.append(scarlar_field)
                scarlar_value.fields = fields

                scarlar_data[key] = scarlar_value

            sdk_param.scalar_data = scarlar_data

        return sdk_param

    def vector_scan(self, scan_param: VectorScanParam) -> list:
        """
        vector_scan scan with start_id

        Args:
            scan_param VectorScanParam

        Raises:
            RuntimeError: return error

        Returns:
            list:  scan info list
        """

        status, out_vector_index = self.client.GetVectorIndex(
            self.schema_id, scan_param.index_name
        )
        if not status.ok():
            raise RuntimeError(
                f"get index {scan_param.index_name} error: {status.ToString()}"
            )
        else:
            if out_vector_index.HasScalarSchema():
                schema_dict = out_vector_index.GetSchema()
                sdk_param = self.vectors_scan_with_schema(schema_dict, scan_param)
            else:
                sdk_param = self.vectors_scan_without_schema(scan_param)

        s, result = self.vector_client.ScanQueryByIndexName(
            self.schema_id, scan_param.index_name, sdk_param
        )

        if s.ok():
            return [
                sdk_vector_with_id_to_vector_with_id(v).to_dict()
                for v in result.vectors
            ]
        else:
            raise RuntimeError(
                f"scan index {scan_param.index_name} fail: {s.ToString()}"
            )

    def get_max_index_row(self, index_name: str) -> int:
        """
        get_max_index_row get max index row

        Args:
            index_name (str): the name of in index

        Raises:
            RuntimeError: return error

        Returns:
            int: max vector id
        """
        s, result = self.vector_client.GetIndexMetricsByIndexName(
            self.schema_id, index_name
        )
        if s.ok():
            metric = sdk_index_metrics_result_to_index_metric(result)
            return metric.max_vector_id
        else:
            raise RuntimeError(f"get index {index_name} metrics fail: {s.ToString()}")

    def vector_search(self, param: VectorSearchParam) -> list:
        """
        vector_search search vector

        Args:
            param : VectorSearchParam

        Raises:
            RuntimeError: return error

        Returns:
            List[dict]: search results
        """
        assert param.fields is not None
        assert param.search_params is not None

        sdk_param = dingosdk.SearchParam()
        sdk_param.topk = param.top_k
        sdk_param.with_vector_data = param.with_vector_data
        sdk_param.with_scalar_data = param.with_scalar_data

        selected_keys = []
        for key in param.fields:
            selected_keys.append(key)
        sdk_param.selected_keys = selected_keys

        # TODO: support vecotr id filter
        sdk_param.filter_source = dingosdk.FilterSource.kScalarFilter

        sdk_param.filter_type = dingosdk.FilterType.kQueryPre
        if not param.pre_filter:
            sdk_param.filter_type = dingosdk.FilterType.kQueryPost

        sdk_param.use_brute_force = param.brute

        extra_params_map = {}
        extra_params_map[dingosdk.SearchExtraParamType.kNprobe] = (
            param.search_params.get("nprobe", constants.N_PROBES)
        )
        extra_params_map[dingosdk.SearchExtraParamType.kRecallNum] = (
            param.search_params.get("recallNum", constants.RECALL_NUM)
        )
        extra_params_map[dingosdk.SearchExtraParamType.kParallelOnQueries] = (
            param.search_params.get("parallelOnQueries", constants.PARALLEL_ON_QUERIES)
        )
        extra_params_map[dingosdk.SearchExtraParamType.kEfSearch] = (
            param.search_params.get("efSearch", constants.EF_SEARCH)
        )

        sdk_param.extra_params = extra_params_map

        if "meta_expr" in param.search_params.keys():
            if param.search_params["meta_expr"] is not None:
                sdk_param.langchain_expr_json = json.dumps(
                    auto_expr_type(
                        convert_dict_to_expr(param.search_params["meta_expr"])
                    ),
                    ensure_ascii=False,
                )
        elif "langchain_expr" in param.search_params.keys():
            if param.search_params["langchain_expr"] is not None:
                sdk_param.langchain_expr_json = json.dumps(
                    auto_expr_type(param.search_params["langchain_expr"]),
                    ensure_ascii=False,
                )

        target_vectors = []
        for xq in param.xq:
            
            if param.value_type == "float" :
                tmp_vector = dingosdk.Vector(dingosdk.ValueType.kFloat, len(xq))
                tmp_vector.float_values = xq
            elif param.value_type=="binary" :
                tmp_vector = dingosdk.Vector(dingosdk.ValueType.kUint8, len(xq))
                tmp_vector.binary_values = xq
            

            tmp = dingosdk.VectorWithId()
            tmp.vector = tmp_vector

            target_vectors.append(tmp)

        s, result = self.vector_client.SearchByIndexName(
            self.schema_id, param.index_name, sdk_param, target_vectors
        )

        if s.ok():
            search_result = [sdk_search_result_to_search_result(v) for v in result]

            empty = True
            for s in search_result:
                if not s.is_empty():
                    empty = False
                    break

            if empty:
                return []
            else:
                return [s.to_dict() for s in search_result]

        else:
            raise RuntimeError(f"search index:{param.index_name} fail: {s.ToString()}")

    def vector_get(self, param: VectorGetParam) -> list:
        """
        vector_get query vector

        Args:
            param: VectorGetParam

        Raises:
            RuntimeError: _description_

        Returns:
            list: _description_
        """
        sdk_param = dingosdk.QueryParam()
        sdk_param.vector_ids = list(set(param.ids))
        sdk_param.with_vector_data = param.with_vector_data
        sdk_param.with_scalar_data = param.with_scalar_data

        # result = dingosdk.QueryResult()
        s, result = self.vector_client.BatchQueryByIndexName(
            self.schema_id, param.index_name, sdk_param
        )
        if s.ok():
            get_results = {
                v.id: sdk_vector_with_id_to_vector_with_id(v) for v in result.vectors
            }

            return_result = []
            for id in param.ids:
                if id in get_results:
                    return_result.append(get_results[id].to_dict())
                else:
                    return_result.append(None)
            return return_result
        else:
            raise RuntimeError(
                f"vector get form index:{param.index_name} fail: {s.ToString()}"
            )

    def vector_delete(self, param: VectorDeleteParam) -> list:
        """
        vector_delete delete vector with ids

        Args:
            param: VectorDeleteParam

        Raises:
            RuntimeError: return error

        Returns:
            list : [True, False, ...]
        """
        s, result = self.vector_client.DeleteByIndexName(
            self.schema_id, param.index_name, param.ids
        )

        if not s.ok():
            raise RuntimeError(
                f"vector delete form index:{param.index_name} fail: {s.ToString()}"
            )

        result_dict = {res.vector_id: res.deleted for res in result}

        delete_status = []
        for id in param.ids:
            assert id in result_dict, f"id: {id} not in result_dict"
            delete_status.append(result_dict[id])

        return delete_status

    def vector_status_by_index(self, index_name: str) -> list[RegionState]:
        """
        vector_status_by_index

        Args:
            index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            list: state list
        """
        s, result = self.vector_client.StatusByIndexName(self.schema_id, index_name)
        if not s.ok():
            raise RuntimeError(
                f"vector status by index form index:{index_name} fail: {s.ToString()}"
            )
        status_state = sdk_state_result_to_state(result)
        return status_state

        # return [s.to_dict() for s in status_state]

    def vector_status_by_region(
        self, param: VectorStatusByRegionIdParam
    ) -> list[RegionState]:
        """
        vector_status_by_region

        Args:
             param: VectorStatusByRegionIdParam

        Raises:
            RuntimeError: return error

        Returns:
            list: state list
        """

        s, result = self.vector_client.StatusByRegionIdIndexName(
            self.schema_id, param.index_name, param.ids
        )

        if not s.ok():
            raise RuntimeError(
                f"vector status by region  form index:{param.index_name} fail: {s.ToString()}"
            )
        status_state = sdk_state_result_to_state(result)
        return status_state

    def vector_build_by_index(self, index_name: str) -> list[RegionStatus]:
        """
        vector_build_by_index

        Args:
            index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """
        s, result = self.vector_client.BuildByIndexName(self.schema_id, index_name)
        if not s.ok() and not s.IsBuildFailed():
            raise RuntimeError(
                f"vector build by index form index:{index_name} fail: {s.ToString()}"
            )
        build_status = sdk_err_status_result_to_err_status(result)
        return build_status

    def vector_build_by_region(
        self, param: VectorBuildByRegionIdParam
    ) -> list[RegionStatus]:
        """
        vector_build_by_index

        Args:
            param: VectorBuildByRegionIdParam

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """

        s, result = self.vector_client.BuildByRegionIdIndexName(
            self.schema_id, param.index_name, param.ids
        )

        if not s.ok() and not s.IsBuildFailed():
            raise RuntimeError(
                f"vector build by region  form index:{param.index_name} fail: {s.ToString()}"
            )
        build_status = sdk_err_status_result_to_err_status(result)
        return build_status

    def vector_load_by_index(self, index_name: str) -> list[RegionStatus]:
        """
        vector_load_by_index

        Args:
            index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """
        s, result = self.vector_client.LoadByIndexName(self.schema_id, index_name)
        if not s.ok() and not s.IsLoadFailed():
            raise RuntimeError(
                f"vector load by index form index:{index_name} fail: {s.ToString()}"
            )
        load_status = sdk_err_status_result_to_err_status(result)
        return load_status

    def vector_load_by_region(
        self, param: VectorLoadByRegionIdParam
    ) -> list[RegionStatus]:
        """
        vector_load_by_region

        Args:
            param: VectorLoadByRegionIdParam

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """

        s, result = self.vector_client.LoadByRegionIdIndexName(
            self.schema_id, param.index_name, param.ids
        )

        if not s.ok() and not s.IsLoadFailed():
            raise RuntimeError(
                f"vector load by region  form index:{param.index_name} fail: {s.ToString()}"
            )
        load_status = sdk_err_status_result_to_err_status(result)
        return load_status

    def vector_reset_by_index(self, index_name: str) -> list[RegionStatus]:
        """
        vector_reset_by_index

        Args:
            index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """
        s, result = self.vector_client.ResetByIndexName(self.schema_id, index_name)
        if not s.ok() and not s.IsResetFailed():
            raise RuntimeError(
                f"vector reset by index form index:{index_name} fail: {s.ToString()}"
            )
        reset_status = sdk_err_status_result_to_err_status(result)
        return reset_status

    def vector_reset_by_region(
        self, param: VectorResetByRegionIdParam
    ) -> list[RegionStatus]:
        """
        vector_reset_by_region

        Args:
            param: VectorResetByRegionIdParam

        Raises:
            RuntimeError: return error

        Returns:
            list: err_status list
        """

        s, result = self.vector_client.ResetByRegionIdIndexName(
            self.schema_id, param.index_name, param.ids
        )

        if not s.ok() and not s.IsResetFailed():
            raise RuntimeError(
                f"vector reset by region  form index:{param.index_name} fail: {s.ToString()}"
            )
        reset_status = sdk_err_status_result_to_err_status(result)
        return reset_status

    def vector_import_add(self, add_param: VectorAddParam) -> list:
        """
        vector_import_add add vector

        Args:
            add_param: VectorAddParam

        Raises:
            RuntimeError: return error

        Returns:
            list: vector id list
        """
        status, out_vector_index = self.client.GetVectorIndex(
            self.schema_id, add_param.index_name
        )
        if not status.ok():
            raise RuntimeError(
                f"get index {add_param.index_name} error: {status.ToString()}"
            )
        else:
            if out_vector_index.HasScalarSchema():
                schema_dict = out_vector_index.GetSchema()
                vectors = self.vectors_add_with_schema(schema_dict, add_param)
            else:
                vectors = self.vectors_add_without_schema(add_param)

        s, vectors = self.vector_client.ImportAddByIndexName(
            self.schema_id, add_param.index_name, vectors
        )

        if s.ok():
            return [sdk_vector_with_id_to_vector_with_id(v).to_dict() for v in vectors]
        else:
            raise RuntimeError(
                f"add vector in {add_param.index_name} fail: {s.ToString()}"
            )

    def vector_import_delete(self, param: VectorDeleteParam):
        """
        vector_import_delete delete vector with ids

        Args:
            param: VectorDeleteParam

        Raises:
            RuntimeError: return error

        Returns:

        """
        s = self.vector_client.ImportDeleteByIndexName(
            self.schema_id, param.index_name, param.ids
        )

        if not s.ok():
            raise RuntimeError(
                f"vector delete form index:{param.index_name} fail: {s.ToString()}"
            )

    def vector_count_memory(self, index_name: str) -> int:
        """
        vector_count_memory  count build vectors

        Args:
           index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            count
        """
        s, result = self.vector_client.CountMemoryByIndexName(
            self.schema_id, index_name
        )
        if s.ok():
            return result
        else:
            raise RuntimeError(
                f"vector count memory form index:{index_name} fail: {s.ToString()}"
            )

    def vector_get_auto_increment_memory(self, index_name: str) -> int:
        """
        vector_get_auto_increment_id get_auto_increment_id

        Args:
           index_name: str

        Raises:
            RuntimeError: return error

        Returns:
            auto_increment_id
        """
        s, result = self.vector_client.GetAutoIncrementIdByIndexName(
            self.schema_id, index_name
        )
        if s.ok():
            return result
        else:
            raise RuntimeError(
                f"vector count memory form index:{index_name} fail: {s.ToString()}"
            )

    def vector_update_auto_increment_id(self, index_name: str, start_id: int):
        """
        vector_update_auto_increment_id update AutoIncrementId

        Args:
           index_name: str

        Raises:
            RuntimeError: return error

        Returns:

        """
        s = self.vector_client.UpdateAutoIncrementIdByIndexName(
            self.schema_id, index_name, start_id
        )
        if not s.ok():
            raise RuntimeError(
                f"vector count memory form index:{index_name} fail: {s.ToString()}"
            )
        
    def vector_dump(self, index_name: str)-> list[str]:
        s,result = self.vector_client.DumpByIndexName(
            self.schema_id, index_name
        )
        
        

        if not s.ok():
            raise RuntimeError(
                f"vector dump form index:{index_name} fail: {s.ToString()}"
            )

        return result


import warnings
from typing import List

from dingodb.protos.proxy_common_pb2 import *
from dingodb.protos.proxy_index_pb2 import *
from pydantic import BaseModel, validator
from dingodb.utils.tools import auto_value_type, auto_expr_type, convert_dict_to_expr
import json
from . import grpc_config as config


class CheckClintParam(BaseModel):
    host: List[str]
    timeout: int


class CheckCreateIndexParam(BaseModel):
    index_name: str
    dimension: int
    index_type: str = "hnsw"
    metric_type: str = "euclidean"
    replicas: int = 3
    index_config: dict = None
    metadata_config: dict = None
    partition_rule: dict = None
    operand: list = None
    auto_id: bool = True
    start_id: int = 1

    @validator("index_type", always=True)
    def check_index_type(cls, value):
        if value not in config.index_config.keys():
            raise Exception(f"index_type  must in {list(config.index_config.keys())}")
        return value

    @validator("metric_type", always=True)
    def check_metric_type(cls, value):
        if value not in config.metric_type.keys():
            raise Exception(f"metric_type  must in {list(config.metric_type.keys())}")
        value = config.metric_type[value]
        return value

    @validator("replicas", always=True)
    def check_replicas(cls, value):
        if value < 0:
            raise Exception(f"{value} must >= 0")
        return value

    @validator("index_config", always=True)
    def check_index_config(cls, value, values):
        index_type = values.get("index_type")
        dimension = values.get("dimension")
        metric_type = values.get("metric_type")
        vector_index_parameter = config.index_config[index_type]

        if value is not None:
            index_keys = vector_index_parameter[
                list(vector_index_parameter.keys())[1]
            ].keys()
            for key, v in value.items():
                if key in index_keys:
                    vector_index_parameter[list(vector_index_parameter.keys())[1]][
                        key
                    ] = v
                    if key == "efConstruction":
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        if v > 500 or v < 100:
                            warnings.warn(f"efConstruction: {v} suggestion in 100-500")
                    if key == "maxElements":
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        if v > 1000000000 or v < 50000:
                            warnings.warn(
                                f"maxElements:{v} suggestion in 50000-1000000000"
                            )
                    if key == "nlinks":
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        if v > 64 or v < 16:
                            warnings.warn(f"nlinks:{v} suggestion in 16-64")

                    if key == "nsubvector":
                        if dimension % v != 0:
                            raise Exception(f"dimension/nsubvector must Divisible")
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                        if v > 256 or v < 4:
                            warnings.warn(f"nsubvector:{v} suggestion in 4-256")
                    if key == "ncentroids":
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                        if v > 4096 or v < 4:
                            warnings.warn(f"ncentroids:{v} suggestion in 4-4096")
                    if key == "bucketInitSize":
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                    if key == "bucketMaxSize":
                        if v <= 0:
                            raise Exception(f"{key} must > 0")
                    if key == "nbitsPerIdx":
                        if v <= 0 or v > 16:
                            raise Exception(f"{key} must > 0 and <=16")

                else:
                    warnings.warn(f"index_config {key} not in {index_type}")

        vector_index_parameter[list(vector_index_parameter.keys())[1]][
            "dimension"
        ] = dimension
        vector_index_parameter[list(vector_index_parameter.keys())[1]][
            "metricType"
        ] = metric_type
        return vector_index_parameter

    @validator("partition_rule", always=True)
    def check_partition_rule(cls, value):
        value = {} if value is None else value
        return value


class CheckVectorAddParam(BaseModel):
    index_name: str
    vectors: List[list]
    datas: List[dict] = None
    ids: List[int] = None

    @validator("datas", always=True)
    def check_datas(cls, value, values):
        value = [{}] * len(values.get("vectors")) if value is None else value
        return value

    @validator("ids", always=True)
    def check_ids(cls, value, values):
        if value is None:
            assert len(values.get("datas")) == len(
                values.get("vectors")
            ), f"length datas:{len(values.get('datas'))} vectors: {len(values.get('vectors'))} is not equal"
        else:
            assert (
                len(values.get("datas")) == len(values.get("vectors")) == len(value)
            ), f"length datas:{len(values.get('datas'))} vectors: {len(values.get('vectors'))} ids:{len(value)} is not equal"
            for id in value:
                if id <= 0:
                    raise Exception("id must > 0")
        return value


class CheckVectorScanParam(BaseModel):
    index_name: str
    start_id: int
    max_count: int = 1000
    is_reverse: bool = False
    without_scalar_data: bool = False
    without_table_data: bool = False
    without_vector_data: bool = False
    fields: list = None
    filter_scalar: dict = None
    end_id: int = 0

    @validator("end_id", pre=True, always=True)
    def check_true_positive_values(cls, value, field):
        if not value >= 0:
            raise ValueError(f"{field.name} must >= 0")
        return value

    @validator("start_id", "max_count", pre=True, always=True)
    def check_positive_values(cls, value, field):
        if not value > 0:
            raise ValueError(f"{field.name} must > 0")
        return value

    @validator(
        "is_reverse",
        "without_vector_data",
        "without_scalar_data",
        "without_table_data",
        pre=True,
        always=True,
    )
    def check_boolean_fields(cls, value):
        return "true" if value else "false"

    @validator("fields", pre=True, always=True)
    def check_fields(cls, value):
        return value or []

    @validator("filter_scalar", pre=True, always=True)
    def check_filter_scalar(cls, value):
        if value is None:
            return {}
        else:
            return value


class CheckVectorSearchParam(BaseModel):
    index_name: str
    xq: list
    index_type: str
    top_k: int = 10
    fields: list = None
    pre_filter: bool = True
    brute: bool = False
    search_params: dict = None

    @validator("xq", always=True)
    def check_xq(cls, value):
        if not isinstance(value[0], list):
            value = [value]
        return value

    @validator("top_k", pre=True, always=True)
    def check_top_k(cls, value, field):
        if not value > 0:
            raise ValueError(f"{field.name} must > 0")
        return value

    @validator("fields", always=True)
    def check_fields(cls, value):
        if value is None:
            value = []
        return value

    @validator("search_params", always=True)
    def check_search_params(cls, search_params, values):
        parameter = VectorSearchParameter(
            selected_keys=values.get("fields"),
            top_n=values.get("top_k"),
            use_brute_force=values.get("brute"),
        )

        with_vector_data = (
            True if search_params is None else search_params.get("withVectorData", True)
        )
        parameter.without_vector_data = not with_vector_data

        with_scalar_data = (
            True if search_params is None else search_params.get("withScalarData", True)
        )
        parameter.without_scalar_data = not with_scalar_data

        vec_search_request = VectorSearchRequest(
            schema_name="dingo", index_name=values.get("index_name")
        )
        use_scalar_filter = False
        if search_params is not None:
            if (
                "meta_expr" in search_params.keys()
                and "langchain_expr" in search_params.keys()
            ):
                raise ValueError(f"meta_expr and langchain_expr cannot coexist")
            elif "meta_expr" in search_params.keys():
                if search_params["meta_expr"] is not None:
                    parameter.vector_filter = SCALAR_FILTER
                    parameter.langchain_expr = json.dumps(
                        auto_expr_type(
                            convert_dict_to_expr(search_params["meta_expr"])
                        ),
                        ensure_ascii=False,
                    )
                    parameter.vector_filter_type = (
                        QUERY_PRE if values.get("pre_filter") else QUERY_POST
                    )
            elif "langchain_expr" in search_params.keys():
                if search_params["langchain_expr"] is not None:
                    parameter.langchain_expr = json.dumps(
                        auto_expr_type(search_params["langchain_expr"]),
                        ensure_ascii=False,
                    )
                    parameter.vector_filter_type = (
                        QUERY_PRE if values.get("pre_filter") else QUERY_POST
                    )

        parameter.use_scalar_filter = use_scalar_filter
        for xq in values.get("xq"):
            vec_with_id = VectorWithId(
                vector=Vector(dimension=len(xq), float_values=xq, value_type=FLOAT)
            )

            vec_search_request.vectors.append(vec_with_id)

        nprobe = 16 if search_params is None else search_params.get("nprobe", 16)
        assert nprobe > 0, f"nprobe must > 0, but get {nprobe}"
        recallNum = (
            100 if search_params is None else search_params.get("recallNum", 100)
        )
        assert recallNum > 0, f"recallNum must > 0, but get {recallNum}"
        index_type = values.get("index_type")
        if index_type == "VECTOR_INDEX_TYPE_HNSW":
            ef_search = (
                32 if search_params is None else search_params.get("efSearch", 32)
            )
            assert ef_search >= 0, f"efSearch must >= 0, but get {ef_search}"
            parameter.hnsw.CopyFrom(SearchHNSWParam(efSearch=ef_search))
        elif index_type == "VECTOR_INDEX_TYPE_FLAT":
            parameter.flat.CopyFrom(
                SearchFlatParam(
                    parallel_on_queries=0
                    if search_params is None
                    else search_params.get("parallelOnQueries", 0)
                )
            )
        elif index_type == "VECTOR_INDEX_TYPE_IVF_FLAT":
            parameter.ivf_flat.CopyFrom(
                SearchIvfFlatParam(
                    parallel_on_queries=0
                    if search_params is None
                    else search_params.get("parallelOnQueries", 0),
                    nprobe=nprobe,
                )
            )
        elif index_type == "VECTOR_INDEX_TYPE_IVF_PQ":
            parameter.ivf_pq.CopyFrom(
                SearchIvfPqParam(
                    nprobe=nprobe,
                    parallel_on_queries=0
                    if search_params is None
                    else search_params.get("parallelOnQueries", 0),
                    recall_num=recallNum,
                )
            )

        vec_search_request.parameter.CopyFrom(parameter)

        return vec_search_request


class CheckVectorGetParam(BaseModel):
    index_name: str
    ids: List[int]
    scalar: bool = True
    vector: bool = True

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        assert len(id_list) > 0, f"ids list length must > 0, but get {id_list}"
        return id_list


class CheckVectorDeleteParam(BaseModel):
    index_name: str
    ids: List[int]

    @validator("ids", always=True)
    def check_ids(cls, value):
        id_list = []
        for id in value:
            if id <= 0:
                raise Exception("id must > 0")
            else:
                id_list.append(id)
        assert len(id_list) > 0, f"ids list length must > 0, but get {id_list}"
        return id_list



from enum import Enum


class ScalarType(Enum):
    BOOL = "BOOL"
    INT64 = "INT64"
    DOUBLE = "DOUBLE"
    STRING = "STRING"


class VectorValueType(Enum):
    FLOAT = "FLOAT"
    BINARY = "BINARY"


class ScalarColumn:
    def __init__(self, key: str, type: ScalarType, speed: bool = False) -> None:
        self.key = key
        self.type = type
        self.speed = speed


class ScalarSchema:
    def __init__(self):
        self.cols = []

    def add_scalar_column(self, col: ScalarColumn):
        self.cols.append(col)


class ScarlarValue:
    def __init__(self, scalar_type: ScalarType, fields: list):
        self.scalar_type = scalar_type
        self.fields = fields

    def to_dict(self):
        return {"fieldType": self.scalar_type.value, "fields": self.fields}


class ScalarData:
    def __init__(self):
        self.data = {}

    def add_scalar_value(self, key: str, scalar_value: ScarlarValue):
        self.data[key] = scalar_value

    def to_dict(self):
        return {key: value.to_dict() for key, value in self.data.items()}


class Vector:
    def __init__(
        self,
        dimension,
        value_type: VectorValueType,
        float_values: list,
        binary_values: list,
    ):
        self.dimension = dimension
        self.value_type = value_type
        self.float_values = float_values
        self.binary_values = binary_values

    def to_dict(self):
        return {
            "dimension": self.dimension,
            "valueType": self.value_type.value,
            "floatValues": [v for v in self.float_values],
            "binaryValues": [v for v in self.binary_values],
        }


class VectorWithId:
    def __init__(self, id: int, vector: Vector, scalar_data: ScalarData):
        self.id = id
        self.vector = vector
        self.scalar_data = scalar_data

    def to_dict(self):
        return {
            "id": self.id,
            "vector": self.vector.to_dict(),
            "scalarData": self.scalar_data.to_dict(),
        }


class MetricType(Enum):
    L2 = "METRIC_TYPE_L2"
    INNER_PRODUCT = "METRIC_TYPE_INNER_PRODUCT"
    COSINE = "METRIC_TYPE_COSINE"
    HAMMING = "METRIC_TYPE_HAMMING"


class VectorWithDistance:
    def __init__(
        self, vector_with_id: VectorWithId, distance: float, metric_type: MetricType
    ):
        self.vector_with_id = vector_with_id
        self.distance = distance
        self.metric_type = metric_type

    def to_dict(self):
        return {
            "id": self.vector_with_id.id,
            "vector": self.vector_with_id.vector.to_dict(),
            "scalarData": self.vector_with_id.scalar_data.to_dict(),
            "distance": self.distance,
            "metricType": self.metric_type.value,
        }


class SearchResult:
    def __init__(self, vector_with_distance_list: list):
        self.vector_with_distance_list = vector_with_distance_list

    def is_empty(self):
        return len(self.vector_with_distance_list) == 0

    def to_dict(self):
        return {
            "vectorWithDistances": [v.to_dict() for v in self.vector_with_distance_list]
        }


class IndexMetric:
    def __init__(
        self,
        index_type: str,
        count: int,
        deleted_count: int,
        max_vector_id: int,
        min_vector_id: int,
        memory_bytes: int,
    ):
        self.index_type = index_type
        self.count = count
        self.deleted_count = deleted_count
        self.max_vector_id = max_vector_id
        self.min_vector_id = min_vector_id
        self.memory_bytes = memory_bytes

    def to_dict(self):
        return {
            "indexType": self.index_type,
            "count": self.count,
            "deletedCount": self.deleted_count,
            "maxVectorId": self.max_vector_id,
            "minVectorId": self.min_vector_id,
            "memoryBytes": self.memory_bytes,
        }


class RegionStatus:
    def __init__(self, region_id: int, err_msg: str):
        self.region_id = region_id
        self.err_msg = err_msg

    def to_dict(self):
        return {"region_id": self.region_id, "err_msg": self.err_msg}




class RegionState:
    def __init__(self, region_id: int, state: str):
        self.region_id = region_id
        self.state = state

    def to_dict(self):
            return {"region_id ": self.region_id, "region_state": self.state}
      


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proxy_meta.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import proxy_common_pb2 as proxy__common__pb2
import proxy_error_pb2 as proxy__error__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10proxy_meta.proto\x12\x15\x64ingodb.pb.proxy.meta\x1a\x12proxy_common.proto\x1a\x11proxy_error.proto\"Q\n\x19PartitionDetailDefinition\x12\x11\n\tpart_name\x18\x01 \x01(\t\x12\x10\n\x08operator\x18\x02 \x01(\t\x12\x0f\n\x07operand\x18\x03 \x03(\t\"v\n\rPartitionRule\x12\x11\n\tfunc_name\x18\x01 \x01(\t\x12\x0f\n\x07\x63olumns\x18\x02 \x03(\t\x12\x41\n\x07\x64\x65tails\x18\x03 \x03(\x0b\x32\x30.dingodb.pb.proxy.meta.PartitionDetailDefinition\"\xf7\x01\n\x0fIndexDefinition\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x02 \x01(\x05\x12=\n\x0findex_partition\x18\x03 \x01(\x0b\x32$.dingodb.pb.proxy.meta.PartitionRule\x12\x0f\n\x07replica\x18\x04 \x01(\x05\x12@\n\x0findex_parameter\x18\x05 \x01(\x0b\x32\'.dingodb.pb.proxy.common.IndexParameter\x12\x1b\n\x13with_auto_increment\x18\x06 \x01(\x08\x12\x16\n\x0e\x61uto_increment\x18\x07 \x01(\x03\"e\n\x12\x43reateIndexRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12:\n\ndefinition\x18\x02 \x01(\x0b\x32&.dingodb.pb.proxy.meta.IndexDefinition\"R\n\x13\x43reateIndexResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\r\n\x05state\x18\x02 \x01(\x08\"e\n\x12UpdateIndexRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12:\n\ndefinition\x18\x02 \x01(\x0b\x32&.dingodb.pb.proxy.meta.IndexDefinition\"R\n\x13UpdateIndexResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\r\n\x05state\x18\x02 \x01(\x08\"Y\n\x18UpdateMaxElementsRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\x12\x14\n\x0cmax_elements\x18\x03 \x01(\x05\"X\n\x19UpdateMaxElementsResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\r\n\x05state\x18\x02 \x01(\x08\"=\n\x12\x44\x65leteIndexRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\"R\n\x13\x44\x65leteIndexResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\r\n\x05state\x18\x02 \x01(\x08\":\n\x0fGetIndexRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\"|\n\x10GetIndexResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12:\n\ndefinition\x18\x02 \x01(\x0b\x32&.dingodb.pb.proxy.meta.IndexDefinition\"(\n\x11GetIndexesRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\"\x7f\n\x12GetIndexesResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12;\n\x0b\x64\x65\x66initions\x18\x02 \x03(\x0b\x32&.dingodb.pb.proxy.meta.IndexDefinition\"+\n\x14GetIndexNamesRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\"T\n\x15GetIndexNamesResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\r\n\x05names\x18\x02 \x03(\t\"A\n\x16GetIndexMetricsRequest\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12\x12\n\nindex_name\x18\x02 \x01(\t\"\xb3\x02\n\x17GetIndexMetricsResponse\x12,\n\x05\x65rror\x18\x01 \x01(\x0b\x32\x1d.dingodb.pb.proxy.error.Error\x12\x12\n\nrows_count\x18\x02 \x01(\x03\x12\x0f\n\x07min_key\x18\x03 \x01(\x0c\x12\x0f\n\x07max_key\x18\x04 \x01(\x0c\x12\x12\n\npart_count\x18\x05 \x01(\x03\x12<\n\nindex_type\x18\x06 \x01(\x0e\x32(.dingodb.pb.proxy.common.VectorIndexType\x12\x15\n\rcurrent_count\x18\x07 \x01(\x03\x12\x15\n\rdeleted_count\x18\x08 \x01(\x03\x12\x0e\n\x06max_id\x18\t \x01(\x03\x12\x0e\n\x06min_id\x18\n \x01(\x03\x12\x14\n\x0cmemory_bytes\x18\x0b \x01(\x03\x32\xd5\x06\n\x0bMetaService\x12\x64\n\x0b\x43reateIndex\x12).dingodb.pb.proxy.meta.CreateIndexRequest\x1a*.dingodb.pb.proxy.meta.CreateIndexResponse\x12\x64\n\x0bUpdateIndex\x12).dingodb.pb.proxy.meta.UpdateIndexRequest\x1a*.dingodb.pb.proxy.meta.UpdateIndexResponse\x12v\n\x11UpdateMaxElements\x12/.dingodb.pb.proxy.meta.UpdateMaxElementsRequest\x1a\x30.dingodb.pb.proxy.meta.UpdateMaxElementsResponse\x12\x64\n\x0b\x44\x65leteIndex\x12).dingodb.pb.proxy.meta.DeleteIndexRequest\x1a*.dingodb.pb.proxy.meta.DeleteIndexResponse\x12[\n\x08GetIndex\x12&.dingodb.pb.proxy.meta.GetIndexRequest\x1a\'.dingodb.pb.proxy.meta.GetIndexResponse\x12\x61\n\nGetIndexes\x12(.dingodb.pb.proxy.meta.GetIndexesRequest\x1a).dingodb.pb.proxy.meta.GetIndexesResponse\x12j\n\rGetIndexNames\x12+.dingodb.pb.proxy.meta.GetIndexNamesRequest\x1a,.dingodb.pb.proxy.meta.GetIndexNamesResponse\x12p\n\x0fGetIndexMetrics\x12-.dingodb.pb.proxy.meta.GetIndexMetricsRequest\x1a..dingodb.pb.proxy.meta.GetIndexMetricsResponseB\x1a\n\x15io.dingodb.proxy.meta\x88\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proxy_meta_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025io.dingodb.proxy.meta\210\001\001'
  _globals['_PARTITIONDETAILDEFINITION']._serialized_start=82
  _globals['_PARTITIONDETAILDEFINITION']._serialized_end=163
  _globals['_PARTITIONRULE']._serialized_start=165
  _globals['_PARTITIONRULE']._serialized_end=283
  _globals['_INDEXDEFINITION']._serialized_start=286
  _globals['_INDEXDEFINITION']._serialized_end=533
  _globals['_CREATEINDEXREQUEST']._serialized_start=535
  _globals['_CREATEINDEXREQUEST']._serialized_end=636
  _globals['_CREATEINDEXRESPONSE']._serialized_start=638
  _globals['_CREATEINDEXRESPONSE']._serialized_end=720
  _globals['_UPDATEINDEXREQUEST']._serialized_start=722
  _globals['_UPDATEINDEXREQUEST']._serialized_end=823
  _globals['_UPDATEINDEXRESPONSE']._serialized_start=825
  _globals['_UPDATEINDEXRESPONSE']._serialized_end=907
  _globals['_UPDATEMAXELEMENTSREQUEST']._serialized_start=909
  _globals['_UPDATEMAXELEMENTSREQUEST']._serialized_end=998
  _globals['_UPDATEMAXELEMENTSRESPONSE']._serialized_start=1000
  _globals['_UPDATEMAXELEMENTSRESPONSE']._serialized_end=1088
  _globals['_DELETEINDEXREQUEST']._serialized_start=1090
  _globals['_DELETEINDEXREQUEST']._serialized_end=1151
  _globals['_DELETEINDEXRESPONSE']._serialized_start=1153
  _globals['_DELETEINDEXRESPONSE']._serialized_end=1235
  _globals['_GETINDEXREQUEST']._serialized_start=1237
  _globals['_GETINDEXREQUEST']._serialized_end=1295
  _globals['_GETINDEXRESPONSE']._serialized_start=1297
  _globals['_GETINDEXRESPONSE']._serialized_end=1421
  _globals['_GETINDEXESREQUEST']._serialized_start=1423
  _globals['_GETINDEXESREQUEST']._serialized_end=1463
  _globals['_GETINDEXESRESPONSE']._serialized_start=1465
  _globals['_GETINDEXESRESPONSE']._serialized_end=1592
  _globals['_GETINDEXNAMESREQUEST']._serialized_start=1594
  _globals['_GETINDEXNAMESREQUEST']._serialized_end=1637
  _globals['_GETINDEXNAMESRESPONSE']._serialized_start=1639
  _globals['_GETINDEXNAMESRESPONSE']._serialized_end=1723
  _globals['_GETINDEXMETRICSREQUEST']._serialized_start=1725
  _globals['_GETINDEXMETRICSREQUEST']._serialized_end=1790
  _globals['_GETINDEXMETRICSRESPONSE']._serialized_start=1793
  _globals['_GETINDEXMETRICSRESPONSE']._serialized_end=2100
  _globals['_METASERVICE']._serialized_start=2103
  _globals['_METASERVICE']._serialized_end=2956
# @@protoc_insertion_point(module_scope)



import proxy_common_pb2 as _proxy_common_pb2
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class Errno(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    OK: _ClassVar[Errno]
    EINTERNAL: _ClassVar[Errno]
OK: Errno
EINTERNAL: Errno

class Error(_message.Message):
    __slots__ = ["errcode", "errmsg"]
    ERRCODE_FIELD_NUMBER: _ClassVar[int]
    ERRMSG_FIELD_NUMBER: _ClassVar[int]
    errcode: Errno
    errmsg: str
    def __init__(self, errcode: _Optional[_Union[Errno, str]] = ..., errmsg: _Optional[str] = ...) -> None: ...



from dingodb.db import DingoDB
from dingodb.grpc import GrpcDingoDB
from dingodb.sdk_vector import SDKVectorDingoDB
from dingodb.sdk_document import SDKDocumentDingoDB
from dingodb.sdk_client import SDKClient

__all__ = [
    "DingoDB",
    "GrpcDingoDB",
    "SDKVectorDingoDB",
    "SDKDocumentDingoDB",
    "SDKClient"
]